"""Generic linear best fits."""

import types
from gmisclib import Num


def fit(idata, fitting_fcns, ff_info=None, wt=None):
	"""Fits  data to a linear combination of fitting_fcns.
	@rtype: tuple
	@return: The returned value is a tuple of:
		the coefficients of the optimal solution (as an array),
		the residual (a single float number),
		the rank of the fit (int),
		an array of the singular values,
		and the best fit to the data.
		The returned value follows Num.LA.linear_least_squares(),
		except that residual is always calculated and is a scalar.
	@type idata: an array of (vector, info).
	@type fitting_fcns: an array of functions.
	@param fitting_fcns: the basis functions for the linear regression.
		The fitting functions return vectors (which must match the data).
		Fitting functions are called as f(info, ff_info, len(vector)).
		Vectors may have different lengths, as long as the
		corresponding data and fit vectors agree.
	"""

	nd = len(idata)
	assert wt is None or len(wt) == nd
	w = [None] * nd

	# We concatenate the data vectors to do one big fit all at once.
	# d_idx keeps track of where each vector starts and ends.
	d_idx = Num.zeros((nd+1,), Num.Int)
	nd_total = 0
	for i in range(nd):
		assert len(idata[i]) == 2
		assert type(idata[i]) == types.TupleType
		d_idx[i] = nd_total		# Where does that dataset start?
		nd_total = nd_total + len(idata[i][0])
	d_idx[nd] = nd_total

	# Build the data array:
	data = Num.zeros((nd_total,), Num.Float)
	for i in range(nd):
		if wt is None:
			w[i] = Num.ones((len(idata[i][0]),), Num.Float)
		else:
			assert len(wt[i]) == len(idata[i][0])
			w[i] = Num.asarray(wt[i], Num.Float)
		assert len(idata[i][0]) == d_idx[i+1]-d_idx[i]
		data[d_idx[i] : d_idx[i+1]] = Num.asarray(idata[i][0],
								Num.Float) * w[i]

	# Build the array of basis functions:
	nff = len(fitting_fcns)
	base = Num.zeros((nd_total, nff), Num.Float)
	for i in range(nd):
		for j in range(nff):
			ff = fitting_fcns[j]
			zfit = Num.asarray(ff(idata[i][1], ff_info, d_idx[i+1]-d_idx[i]),
						Num.Float)
			assert len(zfit.shape)==1
			assert zfit.shape[0] == d_idx[i+1]-d_idx[i]
			base[d_idx[i]:d_idx[i+1], j] = zfit * w[i]

	rcond = 1e-5
	x, resid, rank, s = Num.LA.linear_least_squares(base, data, rcond)
	bfit = Num.matrixmultiply(base, x)
	resid = Num.sum(Num.ravel(bfit-data)**2)

	bestfit = []
	# Chop the fit array back up to match the input vectors:
	for i in range(nd):
		bestfit.append(bfit[d_idx[i]:d_idx[i+1]]/w[i])

	return x, resid, rank, s, bestfit







def test():
	test1()
	test2()
	test3()
	test4()
	test5()
	test6()

def test1():
	x, resid, rank, s, f = fit([([0,1,2,3,4,5], None)], [lambda di, fi, l: range(l)], None)
	assert abs(x[0]-1)<0.0001
	assert resid < 0.001
	assert rank == 1

def test2():
	x, resid, rank, s, f = fit([([0,1,2,3], None), ([0,1,2,3,4], None)], [lambda di, fi, l: range(l)], None)
	assert abs(x[0]-1)<0.0001
	assert resid < 0.001
	assert rank == 1

def test3():
	x, resid, rank, s, f = fit([([0,1,2,3], None), ([0,2,4,6], None)], [lambda di, fi, l: range(l)])
	assert abs(x[0]-1.5)<0.0001
	assert rank == 1
	x, resid, rank, s, f = fit([([0,1,2,3], None), ([0,2,4,6], None)], [lambda di, fi, l: range(l)], wt=[[1,1,1,1],[1e-10,1e-10,1e-10,1e-10]])
	assert abs(x[0]-1)<0.0001
	assert rank == 1

def test4():
	x, resid, rank, s, f = fit([([0,1,2,3], None), ([1,2,3,4], None)], [lambda di, fi, l: range(l), lambda di, fi, l: [1]*l], None)
	assert abs(x[0]-1)<0.0001
	assert abs(x[1]-0.5)<0.0001
	assert rank == 2

def test5():
	x, resid, rank, s, f = fit([([1,2,3,4], None)], [lambda di, fi, l: range(l), lambda di, fi, l: [1]*l])
	assert abs(x[0]-1)<0.0001
	assert abs(x[1]-1)<0.0001
	assert resid < 0.001
	assert rank == 2
	assert abs(f[0][0]-1)<0.001
	assert abs(f[0][1]-2)<0.001

def test6():
	x, resid, rank, s, f = fit(
					[([1,2,3,4], None)],
					[lambda di, fi, l: range(l), lambda di, fi, l: [1]*l],
					wt=[[5,2,3,1e-10]]
				)
	assert abs(x[0]-1)<0.0001
	assert abs(x[1]-1)<0.0001
	assert resid < 0.001
	assert rank == 2





if __name__ == '__main__' :
	test()
	print "OK: passed tests"



"""This a helper module for multivariance.py"""


import multivariance_classes as M
import Num
import random
import gpkmisc

HUGE = 1e30

class quadratic(M.modeldesc):
	__doc__ = """This describes a quadratic model of a known size."""

	def __init__(self, dataset=None, ndim=None):
		assert (dataset is not None) != (ndim is not None)
		if dataset is not None:
			ndim = len(dataset[0])
		M.modeldesc.__init__(self, ndim)


	def modeldim(self):
		m = self.ndim()
		return m + (m*(m+1))/2



	def unpack(self, prms):
		m = self.ndim()
		assert len(prms) == self.modeldim()
		mu = prms[:m]
		invsigma = Num.zeros((m, m), Num.Float)
		j = m
		for i in range(m):
			tmp = prms[j:j+(m-i)]
			invsigma[i,i:] = tmp
			invsigma[i:,i] = tmp
			j += m-i
		return self.new(mu, invsigma)

	def new(self, mu, invsigma, bias=0.0):
		return quadratic_with_numbers(mu, invsigma, self, bias)

	def start(self, data):
		"""Returns (starting position vector, covariance matrix)"""
		n = self.ndim()
		if len(data) > 1:
			iv = M.diag_inv_variance(data)
		else:
			iv = Num.identity(n)

		ivd = Num.diagonal(iv)
		isd = Num.sqrt(ivd)
		size = []
		tmp = []
		for i in range(n):
			for j in range(i,n):
				size.append(isd[i]*isd[j])
			tmp.append(iv[i,i:])
		packedivar = Num.concatenate( tmp )
		# print 'SIZE=', size
		packedvarivar = Num.array( size, copy=True )
		# print 'PIV', packedivar
		# print 'RCD', random.choice(data)
		packedpos = Num.concatenate( ( random.choice(data), packedivar ) )
		packedsize = Num.concatenate( (1.0/Num.diagonal(iv), packedvarivar) )

		return ( packedpos, gpkmisc.make_diag(packedsize) )




class quadratic_with_numbers(M.model_with_numbers):
	def __init__(self, mu, invsigma, details, bias=0.0, offset=None):
		"""self.mu, self.invsigma, and self._offset should be considered
		read-only for all users of this class."""
		assert isinstance(details, quadratic)
		M.model_with_numbers.__init__(self, details, bias)
		n = self.ndim()
		self.mu = Num.array(mu, copy=True)
		assert self.mu.shape == (n,)
		self.invsigma = Num.array(invsigma, copy=True)
		assert self.invsigma.shape == (n,n)
		self._offset = offset

	def __str__(self):
		return '<quadratic: mu=%s; invsigma=%s >' % (str(self.mu), str(self.invsigma))

	__repr__ = __str__

	addoff = M._q_addoff	# Will not be called if _offset is not None



	def logp(self, datum):
		delta = datum - self.mu
		parab = Num.dot(delta, Num.matrixmultiply(self.invsigma,
							delta))
		if not parab >= 0.0:
			raise M.QuadraticNotNormalizable, "Not positive-definite"
		return -parab/2.0 + self.offset() + self.bias

	



class diag_quadratic(quadratic):
	def __init__(self, dataset=None, ndim=None):
		quadratic.__init__(self, dataset, ndim)


	def modeldim(self):
		m = self.ndim()
		return 2*m


	def unpack(self, prms):
		m = self.ndim()
		assert len(prms) == self.modeldim()
		mu = prms[:m]
		invsigma = prms[m:]
		return self.new(mu, invsigma)

	def new(self, mu, invsigma, bias=0.0):
		return diag_quadratic_with_numbers(mu, invsigma, self, bias)

	def start(self, data):
		if len(data) > 1:
			iv = M.vec_inv_variance(data)
		else:
			iv = Num.ones(self.ndim(), Num.Float)

		packeddata = Num.concatenate((random.choice(data), iv))
		packedsize = Num.concatenate((1.0/iv, iv**2))

		return ( packeddata,
			gpkmisc.make_diag(packedsize)
			)



class diag_quadratic_with_numbers(M.model_with_numbers):
	def __init__(self, mu, invsigma, details, bias=0.0, offset=None):
		assert isinstance(details, diag_quadratic)
		M.model_with_numbers.__init__(self, details, bias)
		self.mu = Num.array(mu, copy=True)
		self.invsigma = Num.array(invsigma, copy=True)
		n = self.ndim()
		assert self.invsigma.shape == (n,)
		assert self.mu.shape == (n,)
		self._offset = offset

	def __str__(self):
		return '<diag-quadratic: mu=%s; invsigma=%s >' % (str(self.mu), str(self.invsigma))

	__repr__ = __str__

	addoff = M._d_addoff	# Will not be called if _offset is not None

	def logp(self, datum):
		delta = datum - self.mu
		parab = Num.sum(self.invsigma * delta**2)
		if not parab >= 0.0:
			raise M.QuadraticNotNormalizable, "Not positive-definite"
		return -parab/2.0 + self.offset() + self.bias



Port = 8125
Host = 'localhost'
_TEST = False
import os

try:
	if __name__ == '__main__' or _TEST or 'PYLABDISP' in os.environ:
		raise ImportError, 'testing mode'
	from pylab import *
except ImportError:

	class ProtocolError(Exception):
        	def __init__(self, s):
	                Exception.__init__(self, s)

	class BadPrefixError(ProtocolError):
		def __init__(self, s):
			ProtocolError.__init__(self, s)

	class SocketClosed(ProtocolError):
		def __init__(self):
			ProtocolError.__init__(self, '')





	class _pylab_client(object):
		import socket as _s
		import codecs as _c
		import random as _r
		import cPickle as _p
		import md5 as _m

		ID = 'pylab_server'
		KEY = 'alfaljflajljljslfajsljalsfjalsqowrq nfnqn qkfqle'
		PP = 0

		def wclose(self):
			self.flush()
			self.sock.close()


		def flush(self):
			if self.out:
				# print 'SENDING', self.out
				self.sock.sendall( ''.join(self.out) )
				self.out = []


		def send(self, prefix, s):
			self.out.append( '%s %s\n' % (prefix, self.tohex(s)[0]) )
			# print 'out <= ', self.out[-1], len(self.out)
			self.thash.update(s)
			if len(s) > 1000:
				self.flush()


		def __init__(self, host, port):
			self.fromhex = self._c.getdecoder('hex_codec')
			self.tohex = self._c.getencoder('hex_codec')
			self.thash = self._m.new()
			self.rhash = self._m.new()
			self.thash.update(self.KEY + '1')
			self.rhash.update(self.KEY + '2')
			self.out = []
			self.rqno = None
			self.rfile = None
			self.open(host, port)


		def call(self, fnname, args, argdict):
			self.xmit_call(fnname, args, argdict)
			return self.recv()


		def xmit_call(self, fnname, args, argdict):
			self.send('I', self.ID)
			self.rqno  = self._r.getrandbits(31)
			self.send('R', '%d' % self.rqno)
			self.send('O', 'call')
			self.send('F', fnname)
			self.send('A', self._p.dumps(args, self.PP) )
			self.send('B', self._p.dumps(argdict, self.PP) )
			self.send('H', self.thash.hexdigest() )
			self.send('E', 'END')
			self.flush()
			

		def special(self, op):
			self.xmit_special(op)
			return self.recv()


		def xmit_special(self, op):
			self.send('I', self.ID)
			self.rqno  = self._r.getrandbits(31)
			self.send('R', '%d' % self.rqno)
			self.send('O', op)
			self.send('H', self.thash.hexdigest() )
			self.send('E', 'END')
			self.flush()


		def get(self, prefix):
			tmp = self.rfile.readline()
			if tmp == '':
				raise SocketClosed
			x = self.fromhex(tmp[1:].strip())[0]
			if tmp[0] == 'Z':
				raise ProtocolError, 'Remote: %s' % x
			if tmp[0] != prefix:
				raise BadPrefixError, 'Expected %s, got %s' % (prefix, tmp[0])
			self.rhash.update(x)
			return x


		def recv(self):
			if self.get('i') != self.ID:
				raise ProtocolError, 'ID mismatch'
			if int(self.get('r')) != self.rqno:
				raise ProtocolError, 'Bad rqno.'
			code = self.get('c')
			v = self.get('v')
			localhash = self.rhash.hexdigest()
			if localhash != self.get('h'):
				raise ProtocolError, 'Bad authentication'
			if self.get('e') != 'END':
				raise ProtocolError, 'No end'

			rv = self._p.loads( v )

			if code == 'Exception':
				function, argt, argd, x = rv
				raise Exception(x, function, argt, argd)
			elif code == 'AttributeError':
				raise AttributeError, rv
			return rv


		def open(self, host, port):
			"""Connect to a host.
			The optional second argument is the port number, which
			defaults to the standard telnet port (23).
			Don't try to reopen an already connected instance.
			"""
			self.eof = 0
			self.host = host
			self.port = port
			msg = "getaddrinfo returns an empty list"
			for res in self._s.getaddrinfo(host, port, 0, self._s.SOCK_STREAM):
				af, socktype, proto, canonname, sa = res
				try:
					self.sock = self._s.socket(af, socktype, proto)
					self.sock.connect(sa)
				except self._s.error, msg:
					if self.sock:
						self.sock.close()
						self.sock = None
						continue
				break
			if not self.sock:
				raise self._s.error, '%s on (%s,%s,%s,%s,%s)' % (msg,
							af, socktype, proto, canonname, sa)
			self.rfile = self.sock.makefile('rb', -1)


		def close(self):
			if self.sock:
				# self.special('exit')
				self.sock.close()
				self.sock = None


		def __del__(self):
			self.close()

	def _test():
		c = _pylab_client(Host, Port)
		print c.special('list')[:10]
		print c.call('plot', ([1,2,3,4], [1,2,4,1]), {})
		print c.call('show', (), {})
		c.close()




	if 'PYLABDISP' in os.environ:
		Host, Port = os.environ['PYLABDISP'].split(':')
	if __name__ == '__main__':
		_test()
	else:
		_client = _pylab_client(Host, Port)
		_list = _client.special('list')
		for (_nm, _c) in _list:
			if _c:
				exec("def %s(*a, **b): _client.call('%s', a, b)" % (_nm, _nm))



"""Errors for reading label (typically speech transcription) files.
"""

class Error(Exception):
	def __init__(self, *x):
		Exception.__init__(self, *x)

class NoSuchFileError(IOError, Error):
	def __init__(self, *x):
		IOError.__init__(self, *x)
		Error.__init__(self, *x)

class BadFileFormatError(Error):
	def __init__(self, *x):
		Error.__init__(self, *x)

class DataError(Error):
	def __init__(self, *x):
		Error.__init__(self, *x)

# class DataOutOfOrderError(DataError):
	# def __init__(self, *s):
		# Error.__init__(self, *s)

class DataOutOfOrderError(DataError):
	def __init__(self, *s):
		DataError.__init__(self, *s)



import Num
import math
import die


"""Implements a set of orthonormal polynomials, based on a recurrence relation
on the order of the polynomial.    Each particular type of polynomial just
needs to specify that recurrence relation.
"""


def _orthogonalize(x, others, wt):
	"""Makes x orthonormal to all others.
	Others are assumed to be orthonormal to eachother."""
	against = others[:]
	against.reverse()
	ipass = 0
	x = Num.array(x, Num.Float, copy=True)
	n = x.shape[0]
	while ipass <= 2:
		ipass += 1
		do_all_again = False
		do_again = []
		renorm = True
		for t in against:
			if renorm > 0:
				Num.multiply(x, 1.0/math.sqrt(Num.average( wt * x**2)), x)
				renorm = False
			d = Num.average( x * wt * t)
			# print 'i=', i, 'd=', d
			if abs(d) >= 1e-6:
				do_again.append(t)
				renorm = True

			if abs(d) > 0.1*n:
				do_all_again = True
				if ipass > 1 :
					die.note('x', x)
					die.note('t', t)
					die.note('wt', wt)
					die.note('d', d)
					die.warn('Poor orthogality')
					raise RuntimeError, "Poor orthogonality: %d %g %d/%d" % (len(wt), d, ipass, len(others))
			assert x.shape == t.shape
			# assert type(d) == type(0.1)
			Num.subtract(x, t*d, x)

		if do_all_again:
			against = others
		elif do_again:
			against = do_again
		else:
			break
	if ipass > 2:
		raise RuntimeError, "Can't orthogonalize"
	return x * (1.0/math.sqrt(Num.average(wt*x**2)))


class ortho:
	registry = {}

	def __init__(self, n=None, x=None):
		"""Argument n is how many points you want the polynomials evaluated at.
		The ordinates of the points are placed in self.x.
		You may alternatively specify the points as argument x, in which case
		the x_() function will never be called."""
		if x is not None:
			lx = len(x)
			if not lx > 0:
				raise ValueError, "Need len(x)>0"
			if n is not None:
				assert int(n) == len(x), "X and N mismatch: %d vs %s" % (len(x), n)
			self.x = Num.array(x, Num.Float, copy=True)
		elif n is not None:
			n = int(n)
			if not n>0:
				raise ValueError, "Need n>0"
			self.x = self.x_(n)
		else:
			raise RuntimeError, "Need to specify either x or n."

		self.o = []
		self._wt = None
		self._numwt = None


	def wt(self):
		"""Weighting function to get orthonormality.
		This sets self._numwt."""
		if self._wt is None:
			self._wt = self.wt_()
			self._numwt = Num.sum( self._wt > 0.0 )
			if not Num.alltrue( self._wt >= 0.0 ):
				raise ValueError, 'Weights must be non-negative.'
			if not self._numwt > 0:
				raise ValueError, 'Need some positive weights.'
		# Again, it might make sense to return a copy, as _wt gets used
		# internally after being passed out...
		return self._wt


	def x_(self, m):
		"""Calculates the points at which the function is evaluated,
			if you want m evenly spaced points.
			By default, the function is assumed to
			range over the open interval (-1, 1).
			You can override this function in a subclass
			to get a different range."""
		return (Num.arrayrange(m)-0.5*(m-1))*(2.0/m)


	def P(self, i):
		"""Self.P(i) is the ith orthogonal polynomial.
		The result is normalized so that
		Num.sum(self.wt() * self.P(i)**2) == 1.
		It returns an un-shared array, so the array can be
		modified without affecting future calls to P(i).
		"""
		assert i>=0
		if i<len(self.o):
			return Num.array(self.o[i], copy=True)
		wt = self.wt()	# Sets self._numwt
		if i >= self._numwt:
			raise ValueError, 'Request for higher order polynomial than the number of positive weight points.'
		while i >= len(self.o):
			self.o.append(_orthogonalize(self.compute(len(self.o)), self.o, wt))
		# We return a copy, because it is possible that the user
		# might modify the data.  If so, it would screw up the orthogalization
		# step for higher polynomials.  Copy-on-write would be nice.
		return Num.array(self.o[-1], Num.Float, copy=True)


	def expand(self, c):
		o = Num.zeros(self.x.shape, Num.Float)
		for (i, ci) in enumerate(c):
			tmp = self.P(i)
			Num.multiply(tmp, ci, tmp)
			Num.add(o, tmp, o)
		return o

	def compute(self, n):
		raise RuntimeError, 'Virtual Function'


	def wt_(self):
		raise RuntimeError, "Virtual Function"


class ortho_poly(ortho):
	__doc__ = """Virtual base class.
		This is a superclass of all orthorgonal polynomials.
		This does the recurrence [Abramowitz and Stegun p.782],
		and ensures that the generated polynomials are all
		orthonormal to each other.
		The derived classed need to specify two functions:
		recurse() and wt_().
		Recurse() is the recursion relation from P(i) and P(i-1)
		to P(i+1), and
		wt_() is the weighting function for the polynomial.
		Wt_() is needed to check orthogonality, and really
		defines the polynomial.
		These polynomials are guarenteed to be
		orthogonal to eachother when summed over the
		supplied set of x points.   Thus, if you change
		x_() or wt_(), you get a different set of functions."""


	def __init__(self, n=None, x=None):
		ortho.__init__(self, n, x)


	def P(self, i):
		"""Self.P(i) is the ith orthogonal polynomial.
		The result is normalized so that
		Num.sum(self.P(i)**2) == 1.  """

		assert i>=0
		if i<len(self.o):
			return Num.array(self.o[i], copy=True)
		wt = self.wt()	# Sets self._numwt
		if i >= self._numwt:
			raise ValueError, 'Request for higher order polynomial than the number of positive weight points.'
		while i>=len(self.o):
			if len(self.o)>1:
				m2 = self.o[-2]
			else:
				m2 = None
			if len(self.o)>0:
				m1 = self.o[-1]
			else:
				m1 = None

			tmp = self.recurse(len(self.o), m1, m2)
			self.o.append(_orthogonalize(tmp, self.o, wt))
		# We return a copy, because it is possible that the user
		# might modify the data.  If so, it would screw up the orthogalization
		# step for higher polynomials.  Copy-on-write would be nice.
		return Num.array(self.o[-1], Num.Float, copy=True)


	def recurse(self, n, fn, fnm1):
		"""Does the recurrence relation, evaluating f_(n+1) as a function
		of n, f_n, and f_(n-1).   For n=0 and n=1, fn and fnm1 may be None."""
		raise RuntimeError, "Virtual function"

	def compute(self, n):
		raise RuntimeError, "Compute is not needed and should not be used for subclasses of ortho_poly."


class Legendre(ortho_poly):
	__doc__ = """Legendre polynomials,
		orthonormal over (-1, 1) with weight 1.
		Called Pn(x) in Abramowitz and Stegun.
		"""

	name = 'Legendre'

	def __init__(self, n=None, x=None):
		ortho_poly.__init__(self, n, x)


	def recurse(self, n, fn, fnm1):
		if fn is None:
			assert n == 0
			return Num.ones(self.x.shape, Num.Float)
		if fnm1 is None:
			assert n == 1
			return Num.array(self.x, Num.Float, copy=True)
		# tmp1 = (2*n-1)/float(n)
		# tmp2 = (n-1)/float(n)
		# return tmp1*self.x*fn - tmp2*fnm1
		return self.x*fn - 0.08*fnm1



	def wt_(self):
		return Num.ones(self.x.shape, Num.Float)

ortho.registry[Legendre.name] = Legendre


class Chebyshev(ortho_poly):
	__doc__ = """Chebyshev polynomials of the first kind,
		orthonormal over (-1, 1) with weight (1-x^2)^(-1/2).
		These are the equi-ripple polynomials.
		Called Tn(x) in Abramowitz and Stegun.
		"""

	name = 'Chebyshev'

	def __init__(self, n=None, x=None):
		ortho_poly.__init__(self, n, x)


	def recurse(self, n, fn, fnm1):
		if fn is None:
			assert n == 0
			return Num.ones(self.x.shape, Num.Float)
		if fnm1 is None:
			assert n == 1
			return Num.array(self.x, Num.Float, copy=True)
		# return 2*self.x*fn - fnm1
		return self.x*fn - 0.052*fnm1


	def wt_(self):
		return Num.sqrt(1/(1.0-self.x**2))

ortho.registry[Chebyshev.name] = Chebyshev


class Chebyshev2(ortho_poly):
	__doc__ = """Chebyshev polynomials of the second kind,
		orthonormal over (-1, 1) with weight (1-x^2)^(1/2).
		Called Un(x) in Abramowitz and Stegun.
		"""

	name = 'Chebyshev2'

	def __init__(self, n=None, x=None):
		ortho_poly.__init__(self, n, x)


	def recurse(self, n, fn, fnm1):
		if fn is None:
			assert n == 0
			return Num.ones(self.x.shape, Num.Float)
		if fnm1 is None:
			assert n == 1
			return 2*self.x
		# return 2*self.x*fn - fnm1
		return self.x*fn - 0.445*fnm1


	def wt_(self):
		return Num.sqrt(1.0-self.x**2)

ortho.registry[Chebyshev2.name] = Chebyshev2


class SinCos(ortho):
	__doc__ = """1, sin(pi*x), cos(pi*x), sin(2*pi*x), cos(2*pi*x)...
		orthonormal over (-1, 1) with weight 1."""

	name = 'SinCos'

	def __init__(self, n=None, x=None):
		ortho.__init__(self, n, x)


	def compute(self, n):
		# print 'SinCos.recurse', n
		if n == 0:
			return Num.ones(self.x.shape, Num.Float)
		elif n%2 == 1:
			return Num.sin((math.pi*(n+1)/2) * self.x)
		return Num.cos((math.pi*(n/2))*self.x)


	def wt_(self):
		return Num.ones(self.x.shape, Num.Float)

ortho.registry[SinCos.name] = SinCos





class SLTB(ortho):
	__doc__ = """Smooth Local Trigonometric Basis.
		From Bj\:orn Jawerth, Yi Liu, Wim Sweldens,
		"Signal Compression with Smooth Local Basis Functions".
		"""

	name = 'SLTB'

	def __init__(self, n=None, x=None, eps=0.5):
		assert eps <= 0.5 and eps>0.0
		self.eps = eps
		ortho.__init__(self, n, x)


	def compute(self, n):
		omega = (2*n+1)*math.pi/2.0
		s = Num.sin(omega * self.x)
		return s * self.b(self.x)


	def r(self, x):
		"""This must have l**2+r**2==1"""
		x = Num.clip(x, -self.eps, self.eps)
		s = Num.sin(x * math.pi/(2.0*self.eps))
		r = 1 + s
		l = 1 - s
		return r/Num.hypot(r, l)


	def b(self, x):
		# print 'x=', x
		# print 'b=', self.l(x)*self.l(1.0-x)
		return self.r(x) * self.r(1.0-x)


	def wt_(self):
		return Num.ones(self.x.shape, Num.Float)


	def x_(self, m):
		"""Calculates the points at which the function is evaluated,
			if you want m evenly spaced points.
			The function is assumed to
			range over the open interval (-eps, 1+eps).
			You can override this function in a subclass
			to get a different range."""
		return (0.5+Num.arrayrange(m))*((1+2*self.eps)/m) - self.eps


	def test():
		M = 20
		N = 3*M
		x = (Num.arrayrange(N)+0.5)/float(N) * 3.0 - 0.5
		q = SLTB(x=x)
		for i in range(M//3):
			for j in range(M//3):
				a = q.P(i)
				b = q.P(j)
				# print Num.dot(a[:-M], b[M:])
				assert abs(Num.dot(a[:-M], b[M:])) < 0.001

	test = staticmethod(test)

ortho.registry[SLTB.name] = SLTB



def test(name):
	N = 96
	q = F(name, N)
	for i in range(N//2):
		for j in range(10):
			dot = Num.sum(q.P(i)*q.P(j)*q.wt())
			# print 'p(i=', i, ')', q.P(i)
			# print 'p(j=', j, ')', q.P(j)
			if i != j:
				assert abs(dot) < 0.001
			else:
				assert abs(dot - N) < 0.001*N

	if hasattr(q, 'test'):
		q.test()











def F(name, n=None, x=None):
	"""This is a factory function.  Get any kind of ortho poly
	you want, as selected by the first argument."""

	try:
		return ortho.registry[name](n, x)
	except KeyError:
		raise RuntimeError, "Unimplemented orthogonal function name: %s" % name




if __name__ == '__main__':
	for name in ortho.registry.keys():
		test(name)
	# q = F('SLTB', n=100)
	# for i in range(100):
		# print i, q.P(0)[i]



"""Find different permutations of an array."""

import Num

def _next_guts(a, n):
	"""Increments the array."""
	o = Num.array(a, copy=True)
	o[0] += 1
	for i in range(len(o)):
		if o[i] >= n:
			o[i] = 0
			if i+1 >= n:
				return None
			else:
				o[i+1] += 1
	return o

def _good(p):
	"""Tests if the array is a valid permutation."""
	used = Num.zeros(p.shape, Num.Int)
	for i in p:
		if used[i]:
			return 0
		used[i] = 1
	return 1


def next(a, n):
	"""Finds the first valid permutation of n items after a.
	A valid permutation is an array of length n which
	contains values from 0 to n-1, once each.
	This function returns the lexicographically next permutation;
	it's input need not be a valid permutation, but all the
	entries of a should be in the range 0 to n-1, inclusive.
	"""
	while 1:	# Klugey, slow implementation.
		a = _next_guts(a, n)
		if a is None or _good(a):
			return a
	return None

def factorial(n):
	if n == 1:
		return 1
	return n * factorial(n-1)

def permutations(n):
	"""Returns an array of all the n! permutations of n objects."""
	o = []
	p = Num.zeros((n,), Num.Int)
	if _good(p):
		o.append(p)	# Only for n==0
	while 1:
		p = next(p, n)
		if p is None:
			assert len(o)==factorial(n)
			return o
		o.append(p)
	return None


if __name__ == '__main__':
	assert factorial(5) == 120
	p1 = permutations(1)
	assert len(p1) == 1
	assert len(p1[0]) == 1
	assert p1[0][0] == 0
	p2 = permutations(2)
	assert len(p2)==2
	assert len(p2[0]) == 2
	assert p2[0][0] == 1
	assert p2[0][1] == 0
	assert p2[1][0] == 0
	assert p2[1][1] == 1




"""An extension of mcmc that includes a new stepping algorithm.
"""

from __future__ import with_statement

import math
import random
from gmisclib import die
from gmisclib import mcmc
from gmisclib.mcmc import Debug


def _parab_interp_guts(x0, y0, x1, y1, x2, y2):
	xx01 = x0**2 - x1**2
	x01 = x0 - x1
	y01 = y0 - y1
	xx21 = x2**2 - x1**2
	x21 = x2 - x1
	y21 = y2 - y1
	
	b = (y01*xx21 - y21*xx01) / (xx21*x01 - xx01*x21) 
	a = (y01 - b*x01) / xx01
	c = y0 - a*x0**2 - b*x0
	return (a, b, c)


def pairmax(*xy):
	bestx, besty = xy[0]
	for (x,y) in xy[1:]:
		if y > besty:
			besty = y
			bestx = x
	return (bestx, besty)


def _parab_interp(x0, y0, x1, y1, x2, y2):
	xctr = (x0 + x1 + x2) / 3.0
	# print 'XY=', (x0, y0), (x1, y1), (x2, y2)
	x0 -= xctr
	x1 -= xctr
	x2 -= xctr

	# print 'xy=', (x0, y0), (x1, y1), (x2, y2)
	a, b, c = _parab_interp_guts(x0, y0, x1, y1, x2, y2)
	# print 'abc=', a, b, c
	mn = min(x0, x1, x2)
	mx = max(x0, x1, x2)
	w = mx - mn
	bestx= pairmax((x0,y0), (x1,y1), (x2,y2))[0]
	if a >= 0.0:
		# print 'A>0 !'
		return bestx + w*(random.random()-0.5)
	else:
		xmin = -b/(2*a)
		if min(abs(xmin-x0), abs(xmin-x1), abs(xmin-x2)) < 0.01*w:
			# Too close to replication of an existing point.
			return bestx + w*(random.random()-0.5)
		sigma = math.sqrt(-2.0/a)
		xtmp = random.normalvariate(xmin, sigma)
		# print 'xmin=', xmin, 'sigma=', sigma, '=> xtmp=', xtmp
		if -1.5*w < xtmp < 1.5*w:
			return xctr + xtmp
		return bestx + 1.5*w*(random.random()-0.5)

def test():
	a, b, c = _parab_interp_guts(0.0, 1.0, 1.0, 0.0, 2.0, 1.0)
	assert abs(a-1.0) < 0.001
	assert abs(-b/(2*a) - 1.0) < 0.001
	assert abs(c-1.0) < 0.001


test()

def find_closest_p(v, *vp):
	"""Searches a list of (v,p) pairs and finds the one whose v
	is closest to the first argument.   Returns
	(v,p) of the closest pair.
	"""
	assert len(vp) > 0
	vc, pc = vp[0]
	for (vi, pi) in vp[1:]:
		if abs(vi-v) < abs(vc-v):
			vc = vi
			pc = pi
	return (vc, pc)


class BootStepper(mcmc.BootStepper):
	def __init__(self, lop, v, sort=mcmc.BootStepper.SSAUTO,
				maxArchSize=None, parallelSizeDiv=1):
		mcmc.BootStepper.__init__(self, lop, v, sort=sort,
						maxArchSize=maxArchSize,
						parallelSizeDiv=parallelSizeDiv)
		self.aM = mcmc.adjuster(self.F, self.alpha, vscale=0.5, vse=0.5, vsmax=1.1)


	def step(self):
		mcmc.stepper.step(self)
		leff = min( self.archive.distinct_count(),	# Bootstrapping requires a big archive.
				self.F*self.since_last_rst + self.np_eff,	# Reduce P to 50% after a reset.
				self.PBootLim*self.np_eff		# Keeps P from becoming too large.
				)
		Wboot = leff
		Wmixed = leff
		WV = self.np_eff
		# Parabolic steps are a bootstrap method, so they needs a large archive.
		# But, Step_parab is not really Markov, so we only want to do it 
		# to help find the minimum after a reset.
		Wparab = min(self.archive.distinct_count(), max(0, self.np_eff-self.since_last_rst))
		W = Wboot + Wmixed + Wparab + WV
		Pboot = Wboot/W
		Pmixed = Pboot + Wmixed/W
		Pparab = Pmixed + Wparab/W

		P = random.random()
		try:
			if P < Pboot:
				accepted = self.step_boot()
			elif P < Pmixed:
				accepted = self.step_mixed()
			elif P < Pparab:
					# Note! This violates MCMC requirements!
					# The resulting probability distribution will not be correct
					# when step_parab() is used!
					accepted = self.step_parab()
			else:
				accepted = self.stepV()
		except mcmc.NoBoot:
			accepted = self.stepV()
		return accepted


	def reset_adjusters(self):
		self.aB.reset()
		self.aV.reset()
		self.aM.reset()


	def step_mixed(self):
		self.steptype = 'step_mixed'
		vsm = self.aM.vs()
		vs0 = self.aB.vs() * vsm
		assert vs0 < 30
		vs1 = self.aV.vs() * 0.01 * vsm
		if Debug > 1:
			print 'step_mixed', 'vsm,0,1=', vsm, vs0, vs1

		if len(self.archive) <= 2:
			# print 'NoBoot', len(self.archive)
			raise mcmc.NoBoot
		p1 = self.archive.choose()
		p2 = self.archive.choose()
		if p1.uid() == p2.uid():
			# print 'NoBoot dup'
			raise mcmc.NoBoot

		if self.archive.distinct_count() >= min(5, self.np_eff):
			move = vs1 * (self.archive.variance()/self.v.diagonal())**(1./4.)

		move  = (p1.vec() - p2.vec()) * vs0 + vs1 * self.V.sample()
		tmp = self.current().new(move)
		try:
			delta = tmp.logp() - self.current().logp()
		except mcmc.NotGoodPosition:
			die.warn('NotGoodPosition')
			# self._set_failed( tmp )
			# Should I call self.aM.inctry(accepted) ?
			return 0
		if self.acceptable(delta):
			self._set_failed( None )
			accepted = 1
			self._set_current(tmp)
			self.aM.inctry(accepted)
		else:
			self._set_failed( tmp )
			accepted = 0
			self.aM.inctry(accepted)
			self._set_current(self.current())
		if Debug>2:
			die.info('StepMixed: Accepted logp=%g' % tmp.logp_nocompute())
		return accepted



	def step_parab(self):
		self.steptype = 'step_parab'
		# The parabolic fit will be degenerate if we pick vs0 too close
		# to either of the existing points.
		vs00 = self.aB.vs()
		while True:
			vs0 = random.normalvariate(0.0, vs00)
			if abs(vs0)>0.05 and abs(abs(vs0)-1)>0.05:
				break


		if len(self.archive) <= 2:
			raise mcmc.NoBoot
		p1 = self.current()
		p2 = self.archive.choose()
		if p1.uid() == p2.uid():
			raise mcmc.NoBoot

		vbase, pbase = find_closest_p(vs0, (0.0, p1), (1.0, p2))
		move  = (p2.vec() - p1.vec()) * (vs0-vbase)
		# This is a perfectly good MCMC step, at least when the
		# archive is large and well-annealed.
		tmp = pbase.new(move)
		if Debug > 2:
			print "step_parab: vs0=", vs0, "vbase=", vbase

		try:
			delta = tmp.logp() - self.current().logp()
		except mcmc.NotGoodPosition:
			die.warn('NotGoodPosition')
			# self._set_failed( tmp )
			return 0
		if self.acceptable(delta):
			accepted = 1
			self._set_failed( None )
			self._set_current(tmp)
		else:
			self._set_failed( tmp )
			accepted = 0
			self._set_current(self.current())

		vsnew = _parab_interp(0.0, p1.logp(), 1.0, p2.logp(), vs0, tmp.logp())
		vbase, pbase = find_closest_p(vsnew, (0.0, p1), (1.0, p2), (vs0, tmp))
		if Debug > 2:
			print "step_parab: interp v=", vsnew, "base=", vbase
		move  = (p2.vec() - p1.vec()) * (vsnew-vbase)
		tmp = pbase.new(move)

		try:
			delta = tmp.logp() - self.current().logp()
		except mcmc.NotGoodPosition:
			die.warn('NotGoodPosition')
			# self._set_failed( tmp )
			accepted = 0
		if self.acceptable(delta):
			self._set_failed( None )
			accepted = 1
			self._set_current(tmp)
		else:
			self._set_failed( tmp )
			accepted = 0
			self._set_current(self.current())
		if Debug>2:
			die.info('StepParab: Accepted logp=%g' % tmp.logp_nocompute())
		return accepted



def bootstepper(logp, x, v, c=None, sort=BootStepper.SSAUTO, fixer=None, repeatable=True):
	"""This is (essentially) another interface to the class constructor.
	It's really there for backwards compatibility.
	"""
	pd = mcmc.problem_definition(logp_fcn=logp, c=c, fixer=fixer)
	position_constructor = [mcmc.position_nonrepeatable, mcmc.position_repeatable][repeatable]
	return BootStepper(mcmc.make_list_of_positions(x, position_constructor, pd), v, sort=sort)

diag_variance = mcmc.diag_variance
stepper = mcmc.stepper
problem_definition = mcmc.problem_definition
position_repeatable = mcmc.position_repeatable
position_nonrepeatable = mcmc.position_nonrepeatable
position_history_dependent = mcmc.position_history_dependent


if __name__ == '__main__':
	import sys
	try:
		import psyco
		psyco.full()
	except ImportError:
		pass

	try:
		mcmc.go(sys.argv, bootstepper)
	except:
		die.catch('Unexpected exception')
		raise



"""Markov-Chain Monte-Carlo algorithm.
This is a simulated annealing process which can
be used as an optimizer.

It may be imported as a normal module, and
you then normally create a bootstepper object
and call the step() method to take steps.

Or, you can use it as a script.
If so, you create your own module that implements
a few functions, and run mcmc.py .
It will import your module and call your functions.

The algorithm is not too different from amoeba_anneal, except for two
points:

	1. It keeps lots of points, typically more than 2*N of them, as opposed
   		to amoeba_anneal's N+1.

	2. It switches between two modes of operation:  In minimization mode,
  		it acts much like amoeba_anneal and preferentially tries to step from
  		the best vertex.     In MCMC mode, it starts its step randomly from
  		any vertex.     When it's going down-hill, it switches to
  		minimization mode to go down faster.    When it hasn't found a
  		significant improvement for a long time, it switches into MCMC mode
  		to produce the correct probability distribution.
  

As a script, it takes the following arguments:
	- o fff	Specifies a log file for some basic logging.

	- py mmm/mm	Specifies the module to load and to optimize.
		If the name contains a slash, it adds the base directory
		to the python search path, then looks for a module
		named by the last component.  If no slash, then it
		just looks up the module in PYTHONPATH.
		
	- NI NN	Sets the number of iterations.  Required, unless the
		module defines yourmod.NI.

	- --	Stops interpretation of the argument list.
		The remainder is passed to yourmod.init(), if
		it is defined, then to yourmod.start(), if that is
		defined.

It uses the following functions and variables:

	- yourmod.start(arglist, c)  (Not required).
		This is where you initialize the Monte-Carlo iteration.
		If you define yourmod.start(), it is passed the arglist,
		after the standard mcmc.py flags and arguments are removed
		from it.   Yourmod.start()
		must return a list of one or more starting vectors,
		(either a list of numpy arrays or a list of sequences
		of floats).    These starting vectors are used to seed the iteration.
		
		If it is not defined, mcmc.py will attempt to read a list
		of starting vectors in ascii format from the standard input.
		See mcmc._read().
		
	- yourmod.init(ndim, ni, c, arglist)   (Not required).
		This is primarily there to open a log file.
		Init is passed the remainder of the scripts argument
		list, after yourmod.start() looks at it.    It can do anything
		it wishes with the argument list.
		It can return an object or None.
		
		If it returns something other than None,
		that object will be used to call
		result_of_init.add( ) and
		result_of_init.finish(result_of_init).
		Finish() is there to print a summary or close a log files or similar.
		It is passed the standard output stream.

		Ndim is the length of the parameter vector,
		ni is the number of iterations,
		c is yourmod.c.

	- yourmod.resid(x, c)  (Either yourmod.logp() or yourmod.resid() is required.)

	- yourmod.logp(x, c)  (Either yourmod.logp() or yourmod.resid() is required.)
		These funtions are the thing which is to be optimized.
		Yourmod.log() returns the logarithm (base e) of the probability
		at position x.   X is a Numeric (possibly numarray) vector,
		and c is an arbitrary object that you define.  Often, c contains
		your data.
		
		Yourmod.logp() should return None if x is a silly value.  Note that
		the optimizer will tend to increase values of logp.  It's a maximizer,
		not a minimizer!
		
		If you give yourmod.resid() instead, logp() will be calculated
		as -0.5 times the sum of the squares of the residuals.   It should
		return a Numeric/numarray vector.  It may return None, which will
		cause the optimizer to treat the position as extremely bad.
		
	- yourmod.log(result_of_init, i, p, w)   (Not required).
		This is basically a function to log your data.
		No return value.    It is called at every step and
		passed the result of yourmod.init() (which might
		be a file descriptor to write into).  It is also passed the iteration
		count, i, the current parameter vector, p, and w, which is reserved
		for possible future use.


	- yourmod.NI  (required unless the -NI argument is given on the command line)
		Specifies how many iterations.  This is an integer.

	- yourmod.STARTLOW (not required).  Integer 0 or 1.
		If true, the iteration is started from the largest value of logp
		that is known.

	- yourmod.c	(required).
		This is an arbitrary object that is passed into yourmod.resid() or yourmod.logp().
		Often, it contains data or parameters for these functions,
		but it can be None if you have no need for it.

	- yourmod.V	(Not required)
		This specifies the covariance matrix that is used to hop from one
		test point to another.   It is
		a function that takes the result of yourmod.start() and returns
		a Numeric/numarray 2-dimensional square matrix,

		If it is not specified, mcmc will use some crude approximation
		and will probably start more slowly, but should work OK.
		If it is not specified and yourmod.start() returns only
		one starting position, then it will just use an identity matrix
		as the covariance matrix, and it might still work, but
		don't expect too much.
"""
from __future__ import with_statement

import sys
import math
import random
import Num
import multivariate_normal as MVN
import gpkmisc
import die
import g_implements
import types
import bisect
import threading

Debug = 0

def _logp1(x, c):
	# print "x=", x, "c=", c
	# print "type=", type(x), type(c)
	return -Num.sum(x*x*c*c)


def _logp2(x, c):
	r = Num.identity(x.vec().shape[0], Num.Float)
	r[0,0] = 0.707107
	r[0,1] = 0.707107
	r[1,0] = -0.707107
	r[1,1] = 0.707107
	xt = Num.matrixmultiply(x, r)
	return _logp1(xt, c)


class problem_definition(object):
	"""This class implements the problem to be solved.
	It's overall function in life is to compute the probability that a given
	parameter vector is acceptable.    Mcmc.py then uses that to run a
	Markov-Chain Monte-Carlo sampling.
	Often, you would
	want to derive a class and override all the functions defined here.
	Also, you may well want to define functions of your
	own here.
	
	For instance, it can be a good idea to define a function
	"model" that computes the model that you are fitting to data
	(if that is your plan).   Then logp() can be something
	like -Num.sum((self.model()-self.data)**2).
	Also, it can be good to define a "guess" function that computes
	a reasonable initial guess to the parameters, somehow.
	"""

	@g_implements.make_optional
	@g_implements.make_varargs
	def __init__(self, logp_fcn, c, fixer=None):
		self.c = c
		self._fixer = fixer
		# assert logp_fcn is None or callable(logp_fcn)
		assert callable(logp_fcn)
		self.lpf = logp_fcn

	def logp(self, x):
		"""
		@param x:a parameter vector
		@type x: numpy.ndarray
		@return: The log of the probability that the model assigns to parameters x.
		@rtype: float
		@raise NotGoodPosition: This exception is used to indicate that the position
			x is not valid.   This is equivalent to returning an extremely negative
			logp.
		"""
		return (self.lpf)(x, self.c)
	
	def fixer(self, x):
		"""Fixer() is called on each candidate position
		vector.    Generally, it is used to restrict the possible
		solution space by folding position vectors that escape outside the solution space back into
		the solution space.    It can also allow for symmetries in equations.

		Formally, it defines a convex region.   All vectors outside the region are mapped
		into the region, but the mapping must be continuous at the boundary.
		(More precisely, logp(fixer(x)) must be continuous everywhere that logp(x) is continuous,
		including the boundary.)   For instance, mapping x[0] into abs(x[0]) defines a convex region
		(the positive half-space), and the mapping is continuous near x[0]=0.

		Additionally, it may re-normalize parameters at will subject to the restriction
		that logp(fixer(x))==logp(x).
		For instance, it can implement a constraint that sum(x)==0 by mapping
		x into x-average(x), so long as the value of logp() is unaffected by that
		substitution.
		other folds can sometimes lead to problems.

		@param x:a parameter vector
		@type x: numpy.ndarray
		@return: a (possibly modified) parameter vector.
		@rtype: numpy.ndarray
		@attention: Within a convex region (presumably one that contains the optimal x),
			fixer must *not* change the value of logp(): logp(fixer(x)) == logp(x).

		@raise NotGoodPosition: This exception is used to indicate that the position
			x is not valid.   Fixer has the option of either
			mapping invalid parameter vectors into valid ones
			or raising this exception.
		"""
		if self._fixer is not None:
			return (self._fixer)(x, self.c)
		return x

	@g_implements.make_optional
	def log(self, p, i):
		"""Some code calls this function every iteration
		to log the current state of the MCMC process.
		@param p: the current parameter vector, and
		@param i: an integer iteration counter.
		@return: nothing.
		"""
		pass

	@g_implements.make_optional
	def guess(self):
		"""Some code calls this function at the beginning
		of the MCMC process to seed the iteration.
		@return: a list of guess vectors; each guess vector is
			suitable for passing to self.fixer() or self.logp() as x.
		"""
		pass



_Ntype = type(Num.zeros((1,), Num.Float))



class position_base(object):
	"""This class is used internally in the MCMC sampling process to
	represent a position.   It stores a parameter vector, a reference to
	the problem definition, and (optionally) caches computed values
	of log(P).
	"""
	@g_implements.make_optional
	def __init__(self, x, problem_def):
		"""You need this function and
		this signature if you are going to pass
		it to make_list_of_positions(), but not necessarily
		otherwise.
		"""
		g_implements.check(problem_def, problem_definition)
		self.pd = problem_def
		# print "position:x=", x
		tmp = self.pd.fixer(Num.array(x, Num.Float, copy=True))
		assert isinstance(tmp, _Ntype), "Output of fixer() is not numpy array."
		assert len(tmp.shape) == 1, "Output of fixer() must be 1-dimensional."
		self.x = Num.array(tmp, copy=True)
		self._uid = hash(self.vec().tostring())


	# def __deepcopy__(self, memo):
		# """Don't copy andy deeper than this."""
		# return position_base(self.x, self.pd)


	def logp(self):
		"""Compute the log of the probability for the position.
		"""
		raise RuntimeError, "Virtual Function"


	def logp_nocompute(self):
		"""Shows a recent logp value.  It does not compute
		a value unless none has ever been computed.
		"""
		return self.logp()


	def new(self, shift, logp=None):
		"""Returns a new position, shifted by the specified amount.
		@param shift: How much of a move to make to the new position.
		@type shift: numpy.ndarray
		@param logp: (optional)  If this is supplied, is is used to set the
			log(P) value for the newly created position structure.
		@type logp: float or None
		@return: a new position
		@rtype: L{position_base} or a subclass
		"""
		raise RuntimeError, "Virtual Function"


	def prms(self):
		"""The result of this function is to be handed to problem_definition.logp().
		This result must contain all the information specifying the position.
		Normally, this is a vector of floats, but concievably, you could include other information.
		"""
		return self.x


	def vec(self):
		"""Returns a numpy vector, for mathematical purposes.
		The result should contain all the information specifying the position;
		if not all, it should at least contain all the information that can be
		usefully expressed as a vector of floating point numbers.

		Normally, self.vec() and self.prms() are identical.
		"""
		return self.x


	@g_implements.make_optional
	def __repr__(self):
		return '<POSbase %s>' % str(self.x)


	def __cmp__(self, other):
		"""This is used when the archive is sorted."""
		return cmp(self.logp_nocompute(), other.logp_nocompute())

	def uid(self):
		return self._uid


class position_repeatable(position_base):
	"""This is for the common case where logp is a well-behaved
	function of its arguments.
	"""
	HUGE = 1e38
	CACHE_LIFE = 100
	# CACHE_LIFE = 1	# For debugging only.

	@g_implements.make_optional
	def __init__(self, x, problem_def, logp=None):
		position_base.__init__(self, x, problem_def)
		if logp is not None and not (logp < self.HUGE):
			raise ValueError("Absurdly large value of logP", logp, self.x)
		self.cache = logp
		if logp is None:
			self.cache_valid = -1
		else:
			self.cache_valid = self.CACHE_LIFE


	def invalidate_cache(self):
		"""This can be called when the mapping between parameters (x)
		and value changes.    It's a milder version of position_history_dependent.
		"""
		self.cache_valid = -1


	def __check_cache(self):
		if self.cache_valid <= 0:
			logp = self.pd.logp(self.prms())
			if not (logp < self.HUGE):
				raise ValueError("Absurdly large value of logP", logp, self.x)
			if self.cache_valid==0 and abs(logp - self.cache) > 0.1 + 1e-8*(abs(logp)+abs(self.cache)):
				raise ValueError, 'Recomputing position cache; found mismatch %g to %g' % (self.cache, logp)
			self.cache = logp
			self.cache_valid = self.CACHE_LIFE
		self.cache_valid -= 1
		return self.cache


	def logp(self):
		tmp = self.__check_cache()
		if tmp is None:
			raise NotGoodPosition
		return tmp


	def new(self, shift, logp=None):
		"""Returns a new position, shifted by the specified amount."""
		return position_repeatable(self.vec() + shift, self.pd, logp=logp)


	def __repr__(self):
		if self.cache_valid > 0:
			s = str(self.cache)
		elif self.cache_valid == 0:
			s = "<cache expired>"
		else:
			s = "<uncomputed>"
		return '<POSr %s -> %s>' % (str(self.prms()), s)



class position_nonrepeatable(position_base):
	"""This is for the (unfortunately common) case where logp
	is an indpendent random function of its arguments.
	"""
	HUGE = 1e38

	@g_implements.make_optional
	def __init__(self, x, problem_def, logp=None):
		position_base.__init__(self, x, problem_def)
		if logp is None:
			self.cache = []
		else:
			if not (logp < self.HUGE):
				raise ValueError("Absurdly large value of logP", logp, self.x)
			self.cache = [ logp ]
		self.CSIZE = 6



	def logp(self):
		if random.random() <= self.CSIZE / float(self.CSIZE + len(self.cache)):
			logp = self.pd.logp(self.prms())
			if not (logp < self.HUGE):
				raise ValueError("Absurdly large value of logP", logp, self.x)
			self.cache.append(logp)
		else:
			logp = random.choice(self.cache)
		if logp is None:
			raise NotGoodPosition
		return logp


	def logp_nocompute(self):
		if self.cache:
			tmp = random.choice(self.cache)
		else:
			tmp = self.pd.logp(self.prms())
		if tmp is None:
			raise NotGoodPosition
		return tmp


	def new(self, shift, logp=None):
		"""Returns a new position, shifted by the specified amount."""
		return position_nonrepeatable(self.vec() + shift, self.pd, logp=logp)

	def __repr__(self):
		if len(self.cache):
			s1 = ""
			mn = None
			mx = None
			for q in self.cache:
				if q is None:
					s1 = "BAD or"
				else:
					if mx is None or q>mx:
						mx = q
					if mn is None or q<mx:
						mn = q
			s = "%s%g to %g" % (s1, mn, mx)
		else:
			s = "<uncomputed>"
		return '<POSnr %s -> %s>' % (str(self.prms()), s)


class position_history_dependent(position_base):
	"""This is for the case where logp is a history-dependent
	function of its arguments.   This is the most general, most
	expensive case.
	"""
	EMPTY = "empty"
	HUGE = 1e38

	@g_implements.make_optional
	def __init__(self, x, problem_def, logp=None):
		position_base.__init__(self, x, problem_def)
		if logp is None:
			self.cache = self.EMPTY
		else:
			if not (logp < self.HUGE):
				raise ValueError("Absurdly large value of logP", logp, self.x)
			self.cache = logp



	def logp(self):
		logp = self.pd.logp(self.prms())
		if not (logp < self.HUGE):
			raise ValueError("Absurdly large value of logP", logp, self.x)
		self.cache = logp
		if logp is None:
			return NotGoodPosition
		return logp


	def logp_nocompute(self):
		if self.cache is self.EMPTY:
			self.cache = self.pd.logp(self.prms())
		if self.cache is None:
			return NotGoodPosition
		return self.cache


	def new(self, shift, logp=None):
		"""Returns a new position, shifted by the specified amount."""
		return position_history_dependent(self.vec() + shift, self.pd, logp=logp)


	def __repr__(self):
		if self.cache is self.EMPTY:
			s = "<uncomputed>"
		else:
			s = str(self.cache)
		return '<POSr %s -> %s>' % (str(self.prms()), s)


class T_acceptor(object):
	def __init__(self, T=1.0):
		assert T >= 0.0
		self._T = T

	def T(self):
		return self._T

	def __call__(self, delta):
		return delta > -self._T*random.expovariate(1.0)


class stepper(object):
	"""This is your basic stepper class that incrementally will
	give you a Markov-Chain Monte-Carlo series of samples from
	a probability distribution.
	"""

	@g_implements.make_varargs
	def __init__(self):
		self.since_last_rst = 0
		self.resetid = 0
		self.last_reset = None
		# self.last_failed should reflect the success or failure of the most recently
		# completed step.   It is None if the last step succeeded; it is the
		# most recently rejected position object if the last step failed.
		self.last_failed = None
		self.lock = threading.RLock()
		#: Acceptable is a function that decides whether or not a step is OK.
		#: You can replace it if you want.
		self.acceptable = T_acceptor(1.0)


	def step(self):
		"""In subclasses, this takes a step and returns 0 or 1,
		depending on whether the step was accepted or not."""
		self.since_last_rst += 1
		return None

	def prms(self):
		return self.current().prms()

	def current(self):
		"""Returns the current position.  (An instance of position.)"""
		raise RuntimeError, "Virtual Function"

	def prmlist(self):
		"""Returns all stored positions."""
		raise RuntimeError, "Virtual Function"
	
	def status(self):
		"""Provides some printable status information in a=v; format."""
		raise RuntimeError, "Virtual Function"

	
	def reset(self):
		"""Called internally to mark when the optimization
		has found a new minimum.   [NOTE: You might also call it
		if the function you are minimizing changes.]
		"""
		# print 'stepper.reset', threading.currentThread().getName()
		with self.lock:
			self.since_last_rst = 0
			self.resetid += 1

	def reset_id(self):
		return self.resetid

	def needs_a_reset(self):
		"""Decides if we we need a reset.    This checks
		to see if we have a new record logP that exceeds
		the old record.   It keeps track of the necessary
		paperwork.
		"""
		current = self.current()
		# print 'stepper.needs_a_reset', threading.currentThread().getName()
		with self.lock:
			if self.last_reset is None:
				self.last_reset = current
				rst = False
			else:
				rst = current.logp_nocompute() > self.last_reset.logp_nocompute() + self.acceptable.T()
				# rst = current.logp_nocompute() > self.last_reset.logp_nocompute()
				if rst:
					self.last_reset = current
		if rst and Debug>1:
			print '# RESET: logp=', current.logp_nocompute()
		return rst

	def _set_failed(self, f):
		with self.lock:
			self.last_failed = f



class adjuster(object):
	Z0 = math.log(10.0)
	DZ = 0.3
	TOL = 0.2
	STABEXP = 1.0

	def __init__(self, F, alpha, vscale, vse=0.0, vsmax=1e30):
		assert 0.0 < F < 1.0
		self.lock = threading.Lock()
		self.alpha = alpha
		self.vscale = vscale
		self.F = F
		# self.np = np
		self.state = 0
		self.vse = vse
		self.reset()

		#: Used when the acceptance probability is larger than 25%.
		#: Large acceptance probabilities
		#: can happen if the probability is everywhere about
		#: equal.   (E.g. a data fitting problem with almost
		#: no data):
		self.vsmax = vsmax
		# print "NEW ADJUSTER"

	def reset(self):
		# print 'adjuster.reset', threading.currentThread().getName()
		with self.lock:
			self.z = self.Z0
			self.since_reset = 0
			self.ncheck = self._calc_ncheck()
			self.blocknacc = 0
			self.blockntry = 0
		# print "#ADJUSTER RESET"

	def _calc_ncheck(self):
		# This is the fewest samples where you could possibly
		# detect statistically significant deviations
		# from getting a fraction self.F of steps accepted.
		assert self.z >= self.Z0
		ss =  max( -self.z/math.log(1-self.f()), -self.z/math.log(self.f()) )
		assert ss > 3.0
		return int(math.ceil(max(ss, self.alpha*self.since_reset)))
	
	def f(self):
		"""This describes how the desired fraction of accepted steps should
		depend on self.vscale.   The logic behind this is that the boot
		stepper only works efficiently if self.vscale approx 1.
		If self.vscale < 1, we might as well allow a smaller acceptance probability
		in the hopes of getting a larger vscale.
		This has the side-benefit of avoiding vscale going to zero if (for
		some reason like a corner) the acceptance probability is always
		smaller than self.F.
		"""
		return self.F * min(1.0, self.vscale**self.vse)


	def inctry(self, accepted):
		# print 'adjuster.inctry', threading.currentThread().getName()
		with self.lock:
			self.since_reset += 1
			self.blockntry += 1
			self.blocknacc += accepted
	
			# Only check occasionally.
			if self.blockntry > self.ncheck:
				self._inctry_guts()


	def _inctry_guts(self):
		"""Called under lock!
		We check that the observed fraction of accepted
		steps is consistent with a Binomial distribution.
		If not, we try updating self.vscale.
		"""
		EPS = 1e-6
		Flow = self.f() * (1.0 - self.TOL)
		Fhi = self.f() * (1.0 + self.TOL)
		lPH0low = self.blocknacc*math.log(Flow) + (self.blockntry-self.blocknacc)*math.log(1-Flow)
		lPH0hi = self.blocknacc*math.log(Fhi) + (self.blockntry-self.blocknacc)*math.log(1-Fhi)
		Phat = (self.blocknacc + EPS)/(self.blockntry + 2*EPS)
		sigmaP = math.sqrt(self.f() * (1-self.f()) / self.blockntry)
		lPH1 = self.blocknacc*math.log(Phat) + (self.blockntry-self.blocknacc)*math.log(1-Phat)
		# print '#NCHECK bnacc=', self.blocknacc, self.blockntry, lPH0low, lPH0hi, lPH1, self.z

		if (Phat>Fhi and lPH1-lPH0hi > self.z) or (Phat<Flow and lPH1-lPH0low > self.z):
			# The fraction of accepted steps is inconsistent with the range
			# from Flow to Fhi -- in other words, we're certain the acceptance
			# rate is substantially wrong.

			delta = math.log(Phat/self.f())
			delta = min(max(delta, -self.TOL), self.TOL)
			self.vscale *= math.exp(delta*self.STABEXP)
			if self.vscale > self.vsmax:
				self.vscale = self.vsmax
			if Debug>2:
				print '#NCHECK step acceptance rate is %.2f vs. %.2f:' % (Phat, self.f()), 'changing vscale to', self.vscale
				print '#NCHECK ADJ vscale=', self.vscale, self.z, self.blocknacc, self.blockntry, delta
			self.blockntry = 0
			self.blocknacc = 0
			self.state = -1
			self.ncheck = self._calc_ncheck()
		elif Phat>Flow and Phat<Fhi and 2.0*sigmaP/self.f() < self.TOL:
			# We're as accurate as we need to be.
			# Therefore, we might as well restart the counters
			# in case the process is non-stationary.
			self.blockntry = 0
			self.blocknacc = 0
			self.state = 1
			self.ncheck = self._calc_ncheck()
			# print '#NCHECK close enough', 'phat=', Phat, 'sigmaP=', sigmaP
		else:
			# Doing OK.  The step acceptance rate is not
			# known to be incorrect.
			self.z += self.DZ
			self.state = 0
			self.ncheck = self.blockntry + self._calc_ncheck()
			# print "#NCHECK OK", self.ncheck, 'z=', self.z


	def vs(self):
		"""We stick in the factor of random.lognormvariate()
		so that all sizes of move are possible and thus we
		can prove that we can random-walk to any point in
		a connected region.   This makes the proof of
		ergodicity simpler.
		"""
		assert type(self.vscale)==types.FloatType
		return random.lognormvariate(0.0, self.TOL/2.0)*self.vscale


	def status(self):
		with self.lock:
			tmp = (self.blocknacc, self.blockntry, self.state)
		return tmp






def _start_is_list_a(start):
	"""Is the argument a sequence of numpy arrays?
	"""

	for (i, tmp) in enumerate(start):
		if not isinstance(tmp, _Ntype):
			if i > 0:
				raise TypeError, "Sequence is not all the same type"
			return False
	return len(start) > 0


def _start_is_list_p(start):
	"""Is the argument a sequence of position_base objects?
	"""

	for (i, tmp) in enumerate(start):
		if not g_implements.impl(tmp, position_base):
			if i > 0:
				raise TypeError, "Sequence is not all the same type"
			return False
	return len(start) > 0



class NoBoot(RuntimeError):
	def __init__(self):
		RuntimeError.__init__(self, "Can't find bootstrap points")

class NotGoodPosition(RuntimeError):
	def __init__(self, *s):
		RuntimeError.__init__(self, *s)



def make_list_of_positions(x, PositionClass, problem_def):
	"""Turn almost anything into a list of position_base objects.
	You can hand it a sequence of numpy vectors
	or a single 1-dimensional numpy vector;
	a sequence of position_base objects
	or a single 1-dimensional position_base object.

	@precondition: This depends on PositionClass being callable as
		PositionClass(vector_of_doubles, problem_definition).
	"""
	o = []
	if _start_is_list_a(x):
		assert len(x) > 0, "Zero length list of arrays."
		for t in x:
			if len(o) > 0:
				assert t.shape == (o[0].vec().shape[0],)
			# If the parameters are identical,
			# we can share position structures,
			# and reduce the number of times we need
			# to evaluate logp(x, c).
			if len(o)>0 and Num.alltrue(Num.equal(o[-1].vec(), t)):
				# print 'SAME:', t, self.current()
				o.append( o[-1] )
			else:
				# print 'DIFF:', t
				o.append( PositionClass(t, problem_def) )
	elif _start_is_list_p(x):
		assert len(x) > 0, "Zero length list of positions."
		o = []
		for t in x:
			g_implements.check(t, PositionClass)
			if len(o) > 0:
				assert t.vec().shape[0] == (o[0].vec().shape[0],)
			if len(o)>0 and Num.alltrue(Num.equal(t.vec(), o[-1].vec())):
				o.append( o[-1] )
			else:
				# print 'DIFF:', t
				o.append( t )
	elif g_implements.impl(x, PositionClass):
		o = [x]
	elif isinstance(x, _Ntype) and len(x.shape)==1:
		o = [ PositionClass(x, problem_def) ]
	else:
		raise TypeError, "Cannot handle type=%s for x.  Must implement %s or be a 1-dimensional numpy array." % (type(x), type(PositionClass))
	return o



def _check_list_of_positions(x):
	if not isinstance(x, list):
		raise TypeError, "Not a List (should be a list of positions)"
	for (i, t) in enumerate(x):
		failure = g_implements.why(t, position_base)
		if failure is not None:
			raise TypeError, "x[%d] does not implement position_base: %s" % (i, failure)


class hashcounter_c(dict):
	def incr(self, x):
		try:
			self[x] += 1
		except KeyError:
			self[x] = 1

	def decr(self, x):
		tmp = self[x]
		if tmp == 1:
			del self[x]
		elif tmp < 1:
			raise ValueError("More decrements than increments", x)
		else:
			self[x] = tmp-1





class Archive(object):
	"""This maintains a list of all the recent accepted positions."""
	#: Always keep the list of positions sorted.
	#: This improves mimimization performance at the cost of a distorted probability distribution.
	SSALWAYS = 'always'
	#: Never sort the list.    Best if you really want an exact Monte Carlo distribution.
	SSNEVER = 'neversort'
	#: Sort the list only when logp() is making substantial improvements.
	SSAUTO = 'auto'
	#: Start by sorting positions, then go to SSAUTO.
	SSLOW = 'startlow'

	def __init__(self, lop, np_eff, sort=SSAUTO, maxdups=-1):
		assert sort in (self.SSLOW, self.SSNEVER, self.SSAUTO, self.SSALWAYS)
		assert len(lop) > 0
		self.lop = lop
		self._current = lop[-1]
		self.sortstrategy = sort
		self.sorted = sort in [self.SSLOW, self.SSALWAYS]
		if self.sorted:
			self.sort()
		self.np_eff = np_eff
		self.since_last_reset = 0
		self.lock = threading.Lock()
		if maxdups > 0:
			self.maxdups = maxdups
		else:
			self.maxdups = 100000
		self._hashes = hashcounter_c()
		for p in self.lop:
			self._hashes.incr(p.uid())


	def distinct_count(self):
		with self.lock:
			return len(self._hashes)
	

	def prmlist(self):
		with self.lock:
			return list(self.lop)


	def sort(self):
		"""Called under lock."""
		if self.sorted or self.sortstrategy==self.SSNEVER:
			return
		if Debug:
			die.info( '# Sorting archive' )
		# This uses position.__cmp__():
		self.lop.sort()
		self.sorted = True


	def reset(self):
		"""We sort the archive to speed the convergence to
		the best solution.   After all, if you've just
		gotten a reset, it is likely that you're not at the
		bottom yet, so statistical properties of the distribution
		are likely to be irrelevant.
		"""
		# print 'Archive.reset', threading.currentThread().getName()
		with self.lock:
			self.sort()
			self.since_last_reset = 0
	

	def __len__(self):
		with self.lock:
			return len(self.lop)


	def current(self):
		with self.lock:
			return self._current

	def choose(self):
		# print 'Archive.lock', threading.currentThread().getName()
		with self.lock:
			return random.choice( self.lop )


	def truncate(self, desired_length):
		"""Desired length is in terms of the number of distinct
		positions.
		"""
		with self.lock:
			assert len(self.lop) > 0
			assert len(self._hashes) <= len(self.lop)
			assert (len(self._hashes)>0) == (len(self.lop)>0)

			j = 0
			while len(self._hashes) > desired_length:
				self._hashes.decr(self.lop[j].uid())
				j += 1
			assert j <= len(self.lop)
			if j > 0:
				self.truncate_hook(self.lop[:j])
				self.lop = self.lop[j:]
				if Debug > 1:
					die.info('Truncating archive from %d by %d' % (len(self.lop), j))
			elif Debug > 1:
				die.info('Truncate: max=%d, len=%d, nothing done' % (desired_length, len(self.lop)))
			assert len(self._hashes) <= len(self.lop)
			# There is a possibility that we have truncated the current position.
			# I wonder if that's a problem?


	def _too_many_dups(self, uid):
		"""Called under lock."""
		if not self.lop or self.maxdups<0:
			return False
		st = max(len(self.lop) - self.maxdups, 0)
		for i in range(st, len(self.lop)):
			if self.lop[i].uid() != uid:
				return False
		return True


	def append(self, x, forbid_dups):
		"""Adds stuff to the archive, possibly sorting the
		new information into place.   It updates all kinds of
		counters and summary statistics.

		@param x: A position to (possibly) add to the archive.
		@type x: L{position_base}
		@param forbid_dups: Don't add duplicates to the archive.
		@type forbid_dups: boolean
		@return: A one-letter code indicating what happened.  'f' if x
			is a duplicate and duplicates are forbidden.
			'd' if it is a duplicate and there have been
			too many duplicates lately.
			'a' otherwise -- x has been added to the archive.
		@rtype: str
		"""
		# print 'Archive.append', threading.currentThread().getName()
		with self.lock:
			self._current = x
			self.since_last_reset += 1

			assert len(self._hashes) <= len(self.lop)
			uid = x.uid()
			if forbid_dups and uid in self._hashes:
				return 'f'
			if self._too_many_dups(uid):
				# We don't want the clutter up the archive
				# with duplicates until the step sizes are
				# well established.
				if Debug>1:
					print 'Too many duplicates (more than %d): %s not archived.' % (
						self.maxdups, uid
						)
				assert len(self._hashes) <= len(self.lop)
				return 'd'
			self._hashes.incr(uid)

			if not self.sorted or self.sortstrategy==self.SSNEVER:
				self.lop.append(x)
				self.sorted = False
			else:
				# This uses position.__cmp__():
				bisect.insort_right(self.lop, x)
				# We can't do the following check because (a) it can
				# fail for position_nonrepeatable, and (b) it triggers
				# extra evaluations of logp(), which can be expensive:
				# assert self.lop[-1].logp() >= self.lop[0].logp()
				if self.since_last_reset > self.np_eff and self.sortstrategy!=self.SSALWAYS:
					self.sorted = False
					if Debug:
						die.info( '# Archive sorting is now off' )
			self.append_hook(x)
			if Debug > 1:
				die.info('Archive length=%d' % len(self.lop))
			# print "self.lop=", self.lop
			assert len(self._hashes) <= len(self.lop)
		return 'a'


	def append_hook(self, x):
		pass

	def truncate_hook(self, to_be_dropped):
		pass


class ContPrmArchive(Archive):
	def __init__(self, lop, np_eff, sort=Archive.SSAUTO, maxdups=-1):
		"""Append_hook() is called for every element of the archive.
		That function can be replaced in a sub-class to accumulate
		some kind of summary.  Here, it is used to keep track of parameter
		means and standard deviations.
		"""
		Archive.__init__(self, lop, np_eff, sort=sort, maxdups=maxdups)
		self.p0 = lop[-1].vec()
		self.s = Num.zeros(self.p0.shape, Num.Float)
		self.ss = Num.zeros(self.p0.shape, Num.Float)
		for a in lop:
			self.append_hook(a)


	def append_hook(self, x):
		"""This accumulates parameter means and standard deviations.
		"""
		tmp = x.vec() - self.p0
		Num.add(self.s, tmp, self.s)
		Num.add(self.ss, tmp**2, self.ss)


	def truncate_hook(self, to_be_dropped):
		if len(to_be_dropped) > len(self.lop):
			# Take the opportunity to pick a better value of self.p0.
			self.p0 = self.s/(len(to_be_dropped) + len(self.lop))
			self.s = Num.zeros(self.p0.shape, Num.Float)
			self.ss = Num.zeros(self.p0.shape, Num.Float)
			for prms in self.lop:
				self.append_hook(prms)
		else:
			for prms in to_be_dropped:
				tmp = prms.vec() - self.p0
				Num.subtract(self.s, tmp, self.s)
				Num.subtract(self.ss, tmp**2, self.ss)

	def variance(self):
		with self.lock:
			n = len(self.lop)
			core = self.ss - self.s*self.s/n
			if not Num.alltrue(Num.greater(core, 0.0)):
				self.ss -= Num.minimum(0.0, core)
				if Debug > 0:
					die.warn('Zero stdev in archive.stdev for p=%s'
							% ','.join( ['%d' % q for q
								in Num.nonzero(Num.less_equal(core, 0.0))[0]
							])
						)
				return Num.ones(self.s.shape, Num.Float)
		assert n > 1
		assert Num.alltrue(Num.greater(core, 0.0))
		return core/(n-1)


class BootStepper(stepper):
	#: Targeted step acceptance rate.
	F = 0.25
	alpha = 0.1
	#: Limits the probability of taking a bootstrap step to 1-1/PBootLim = 90%:
	PBootLim = 10.0
	#: Control of when to sort steps and when to run with honest Monte-Carlo:
	SSAUTO = ContPrmArchive.SSAUTO
	SSNEVER = ContPrmArchive.SSNEVER
	SSLOW = ContPrmArchive.SSLOW
	SSALWAYS = ContPrmArchive.SSALWAYS

	def __init__(self, lop, v, sort=SSAUTO, maxArchSize=None, parallelSizeDiv=1):
		"""@param maxArchSize: How many position vectors can be stored.  This is
			normally used to (loosely) enforce a memory limitation for large
			jobs.
		@param parallelSizeDiv: For use when there are several cooperating MCMC
			processes that share data.  When >1, this allows each process
			to have smaller stored lists.   Normally, parallelSizeDiv is
			between 1 and the number of cooperating processes.
		"""
		stepper.__init__(self)
		_check_list_of_positions(lop)
		#: The number of parameters:
		self.np = lop[0].vec().shape[0]
		#: In a multiprocessor situation, np_eff tells you how much data do you
		#: need to store locally, so that the overall group of processors
		#: stores enough variety of data.
		self.np_eff = self.np//parallelSizeDiv
		self.archive = ContPrmArchive(lop, self.np_eff, sort=sort, maxdups=int(round(4.0/self.F)))

		if not self.np > 0:
			raise ValueError, "Np=%d; it must be positive." % self.np
		if v.shape != (self.np, self.np):
			raise ValueError, "v must be square, with side equal to the number of parameters. Vs=%s, np=%d." % (str(v.shape), self.np)

		self.v = Num.array(v, Num.Float, copy=True)
		self.V = MVN.multivariate_normal(Num.zeros(v.shape[0], Num.Float),
								v)
		self.aB = adjuster(self.F, self.alpha, vscale=0.5, vse=0.5, vsmax=1.3)
		self.aV = adjuster(self.F, self.alpha, vscale=1.0)

		if maxArchSize is None:
			maxArchSize = 2*self.np**2 + 1000	# Big!
		if maxArchSize < self.np//parallelSizeDiv + 7:
			die.warn("maxArchSize is too small for trustworthy operation: %d vs. %d" % (maxArchSize, self.np))
		self.min_l = min(maxArchSize,
				(self.np//parallelSizeDiv+maxArchSize)//2+7,
				(self.np*2)//parallelSizeDiv+7
				)
		self.max_l = min(maxArchSize, (self.np**2)//parallelSizeDiv+self.min_l)
		self.steptype = None


	def step(self):
		stepper.step(self)
		leff = min( self.archive.distinct_count(),	# Bootstrapping requires a big archive.
				self.F*self.since_last_rst + self.np_eff,	# Reduce P to 50% after a reset.
				self.PBootLim*self.np_eff		# Keeps P from becoming too large.
				)
		P = leff / float(leff + self.np_eff)
		if random.random() < P:
			try:
				accepted = self.step_boot()
			except NoBoot:
				accepted = self.stepV()
		else:
			accepted = self.stepV()
		return accepted



	def current(self):
		return self.archive.current()

	def prmlist(self):
		return self.archive.prmlist()

	# def last(self):
		# return self.archive.last()
		
	
	def status(self):
		with self.lock:
			o = ['a0vs=%g' % self.aB.vscale,
				'a0acc=%g; a0try=%g; a0state=%d' % self.aB.status(),
				'a1vs=%g' % self.aV.vscale,
				'a1acc=%g;  a1try=%g; a1state=%d' % self.aV.status(),
				'nboot=%d' % len(self.archive),
				'logP=%g' % self.current().logp_nocompute(),
				'type=%s' % self.steptype
				]
		return '; '.join(o) + ';'


	def stepV(self):
		self.steptype = 'stepV'
		vs1 = self.aV.vs()
		move = self.V.sample() * vs1
		if hasattr(self.archive, 'variance') and self.archive.distinct_count() >= min(5, self.np):
			move *= (self.archive.variance()/self.v.diagonal())**0.25

		# print "self.current()=", self.current()
		try:
			tmp = self.current().new(move)
			delta = tmp.logp() - self.current().logp()
		except NotGoodPosition:
			if Debug>2:
				die.warn('StepV: NotGoodPosition')
			# self._set_failed( tmp )
			# Should I call self.aV.inctry(0) ?
			return 0
		if self.acceptable(delta):
			self._set_failed( None )
			accepted = 1
			if Debug>2:
				die.info('StepV: Accepted logp=%g' % tmp.logp_nocompute())
			# We cannot use self.lop[-1] for current because lop is
			# sometimes sorted.  When it is, the most recently added
			# position could end up anywhere in the list.   Thus, we keep
			# a separate self._current.
			self._set_current(tmp)
			self.aV.inctry(accepted)
		else:
			self._set_failed( tmp )
			accepted = 0
			if Debug>2:
				die.warn('StepV: Rejected logp=%g' % tmp.logp_nocompute())
			self.aV.inctry(accepted)
			self._set_current(self.current())
		return accepted


	def step_boot(self):
		self.steptype = 'step_boot'
		vs0 = self.aB.vs()
		assert vs0 < 10.0, "Vs0 too big!=%s" % str(vs0)
		if Debug>3:
			die.note('VsB', vs0)

		if len(self.archive) <= 2:
			# print 'NoBoot', len(self.archive)
			raise NoBoot
		p1 = self.archive.choose()
		p2 = self.archive.choose()
		# if p1 is p2:
		if p1.uid() == p2.uid():
			# print 'NoBoot dup'
			raise NoBoot

		# die.note('p1.prms', p1.prms())
		# die.note('p2.prms', p2.prms())
		move  = (p1.vec() - p2.vec()) * vs0
		# print '# move=', p1.prms(), p2.prms(), vs0, move

		try:
			tmp = self.current().new(move)
			delta = tmp.logp() - self.current().logp()
		except NotGoodPosition:
			if Debug>2:
				die.warn('StepBoot: Nogood position')
			# self._set_failed( tmp )
			return 0
		if self.acceptable(delta):
			self._set_failed( None )
			accepted = 1
			if Debug>2:
				die.info('StepBoot: Accepted logp=%g' % tmp.logp_nocompute())
			self._set_current(tmp)
			self.aB.inctry(accepted)
		else:
			self._set_failed( tmp )
			accepted = 0
			if Debug>2:
				die.warn('StepBoot: Rejected logp=%g' % tmp.logp_nocompute())
			self.aB.inctry(accepted)
			self._set_current(self.current())
		return accepted



	def _set_current(self, newcurrent):
		g_implements.check(newcurrent, position_base)
		# print 'bootstepper.set_current', threading.currentThread().getName()
		out = []
		with self.lock:
			#: After a reset, the optimizer may not have good step directions,
			#: so there may be many failed steps.   Thus, you don't want to
			#: fill up the archive with lots of duplicates of the current position
			#: that are basically meaningless.    Thus, we forbid duplicates for
			#: a while.
			forbid_dups = self.since_last_rst < self.np_eff
			q = self.archive.append(newcurrent, forbid_dups=forbid_dups)
			out.append(q)
			if self.needs_a_reset():
				# Shorten the archive -- if we've found a new best
				# point, that makes it likely that we have not
				# yet converged.   Consequently, since were
				# probably not near stationarity, the old history
				# is probably useless.
				self.reset_adjusters()
				self.archive.reset()
				self.reset()
				out.append('r')
			self.archive.truncate( min(int( self.min_l + self.F*self.alpha*self.since_last_rst ),
						self.max_l)
					)
		return ''.join(out)


	def reset_adjusters(self):
		self.aB.reset()
		self.aV.reset()


	def ergodic(self):
		"""A crude measure of how ergodic the MCMC is.
		This is the inverse of how many steps it takes to cross the
		minimum in the slowest direction.
		"""
		if self.since_last_rst*self.aB.f() < self.np_eff:
			# Presumably, we are still stabilizing.
			# print '# SLR in ergodic'
			return 0.0
		if self.aB.state < 0:
			# The step size is known to be mis-adjusted,
			# so you cannot use it to compute a measure
			# of the ergodicity.
			# print '# state<0 in ergodic', self.aB.state
			return 0.0

		# This is calibrated from the equilibrium vscale from long runs
		# in 1-50 dimensional Gaussian problems.

		vsbar = 1.7/math.sqrt(self.np)	# Should this be np_eff ?
		vs = self.aB.vscale
		# print 'ergodic: vs=', vs, "erg=", min(1.0, vs/vsbar)**2
		return self.F * min(1.0, vs/vsbar) ** 2


	def set_sort_strategy(self, ss):
		tmp = self.archive.sortstrategy
		self.archive.sortstrategy = ss
		return tmp


def bootstepper(logp, x, v, c=None, sort=BootStepper.SSAUTO, fixer=None, repeatable=True):
	"""This is (essentially) another interface to the class constructor.
	It's really there for backwards compatibility.
	"""
	pd = problem_definition(logp_fcn=logp, c=c, fixer=fixer)
	position_constructor = [position_nonrepeatable, position_repeatable][repeatable]
	return BootStepper(make_list_of_positions(x, position_constructor, pd), v, sort=sort)

boot2stepper = bootstepper


def test():
	logp = _logp1
	# logp = _logp2
	start = [1.0, 2.0, 2, 9, 1]
	c = [10.0, 10.0, 10.0, 10.0, 0.1]
	V = Num.identity(len(c), Num.Float)
	# V = Num.array([[ 20.69626808,  20.6904984 ], [ 20.6904984,   20.69477235]])

	x = bootstepper(logp, start, V, c=c)

	v = Num.zeros(V.shape, Num.Float)
	for i in range(10000):
		x.step()
		p = x.vec()
		sys.stdout.writelines('%d ' % i)
		for j in range(x.vec().shape[0]):
			sys.stdout.writelines('%g ' % p[j])
		sys.stdout.writelines('\n')
		v += Num.outerproduct(p, p)
	
	v /= 10000.0
	sys.stderr.writelines(str( v ) + "\n")
	evalues, evec = Num.LA.eigh(v)
	sys.stderr.writelines("Eval: %s\n" % str(evalues))
	sys.stderr.writelines("Evec: %s\n" % str(evec))


def _print(fd, prefix, p):
	fd.writelines( ' '.join( [prefix] + [ '%g'%pj for pj in p ] ) )
	fd.writelines('\n')


def _read(fd):
	o = []
	for t in fd:
		if t.startswith('#'):
			continue
		t = t.strip()
		if t == '':
			continue
		o.append( Num.array( [float(x) for x in t.split() ],
					Num.Float)
			)
	np = o[-1].shape[0]
	# die.info("np=%d" % np)
	return o[ max(0, len(o)-2*np*np) : ]





def diag_variance(start):
	"""Hand this a list of vectors and it will compute
	the variance of each component, then return a diagonal
	covariance matrix.
	"""
	tmp = gpkmisc.vec_variance(start)
	if not Num.alltrue(Num.greater(tmp, 0.0)):
		raise ValueError, ("Zero variance for components %s"
					% ','.join(['%d'%q for q in
							Num.nonzero(1-Num.greater(tmp, 0.0))[0]
						]
						)
					)
	return gpkmisc.make_diag(tmp)


def logp_from_resid(x, c, resid_fcn):
	r = resid_fcn(x, c)
	if r is None:
		raise NotGoodPosition
	# return -Num.sum(Num.ravel(r)**2)/2.0
	return -Num.sum(r**2)/2.0


def go(argv, theStepper):
	"""Run the optimization, controlled by command line flags."""
	global Debug
	import load_mod
	if len(argv) <= 1:
		print __doc__
		return
	python = None
	out = None
	NI = None
	arglist = argv[1:]
	while len(arglist) > 0  and arglist[0][0] == '-':
		arg = arglist.pop(0)
		if arg == '-o':
			out = arglist.pop(0)
		elif arg == '-py':	# path/module
			python = arglist.pop(0)
		elif arg == '-NI':
			NI = int(arglist.pop(0))
		elif arg == '-debug':
			Debug += 1
		elif arg == '--':
			break
		else:
			die.die("Unrecognized flag: %s" % arg)
	
	assert python is not None, "Need to use the -py flag."

	mod = load_mod.load_named_module(python)
	print "# mod= %s" % str(mod)

	if out:
		logfd = open(out, "w")
	else:
		logfd = None

	mod_c = getattr(mod, "c", None)
	if not hasattr(mod, "start"):
		start = _read(sys.stdin)
	else:
		start = mod.start(arglist, mod_c)

	if hasattr(mod, 'logp'):
		logPfcn = mod.logp
	elif hasattr(mod, 'resid'):
		logPfcn = lambda x, c, m=mod: logp_from_resid(x, c, m.resid)
	else:
		die.die("Can't find logp() or resid() function in module.")

	if NI is None:
		NI = mod.NI
	print "# NI= %d" % NI


	mod_c = getattr(mod, "c", None)	#A kluge, in case mod.start() changed c.
	if hasattr(mod, "V"):
		V = Num.asarray(mod.V(start, mod_c), Num.Float)
	elif _start_is_list_a(start) and len(start)>1:
		V = diag_variance(start)
	else:
		V = Num.identity(start[0].shape[0], Num.Float)

	mod_c = getattr(mod, "c", None)	#A kluge, in case mod.V() changed c.
	if hasattr(mod, "init"):
		logptr = mod.init(V.shape[0], NI, mod_c, arglist)
	else:
		logptr = def_logger(V.shape[0], NI, mod_c, arglist)

	sort = getattr(mod, 'SORT_STRATEGY', BootStepper.SSAUTO)

	fixer = getattr(mod, 'fixer', None)

	mod_c = getattr(mod, "c", None)	#A kluge, in case mod.init() changed c.
	sys.stdout.flush()
	x = theStepper(logPfcn, start, V, c=mod_c, sort=sort, fixer=fixer)

	sys.stdout.writelines('[P]\n')
	for i in range(NI):
		x.step()
		p = x.prms()
		sys.stdout.flush()
		if logfd is not None:
			logfd.writelines('# ' + x.status() + '\n')
			_print(logfd, '%d'%i , p)
			logfd.flush()
		if logptr is not None:
			logptr.add(p, i)
	if logptr is not None:
		logptr.finish(sys.stdout)


class def_logger:
	def __init__(self, ndim, ni, c, arglist):
		self.v = Num.zeros((ndim,ndim), Num.Float)
		self.n = 0
		self.NI = ni
		self.c = c
	
	def add(self, p, i):
		if i >= self.NI/10:
			self.v += Num.outerproduct(p, p)
			self.n += 1

	def finish(self, stdout):
		self.v /= float(self.n)
		sys.stdout.writelines('[V]\n')
		sys.stderr.writelines(str( self.v ) + "\n")
		evalues, evec = Num.LA.eigh(self.v)
		stdout.writelines('[Evals]\n')
		stdout.writelines("%s\n" % str(evalues))
		stdout.writelines('[Evecs]\n')
		stdout.writelines("%s\n" % str(evec))






if __name__ == '__main__':
	try:
		import psyco
		psyco.full()
	except ImportError:
		pass

	try:
		go(sys.argv, bootstepper)
	except:
		die.catch('Unexpected exception')
		raise




import Numeric

class NoSolutionError(ValueError):
	def __init__(self, s):
		ValueError.__init__(self, s)

def solve(a, b0):
	""" solves a*x = b."""
	b = Numeric.asarray(b0, Numeric.Float)
	if len(b.shape) == 1:
		b = Numeric.array((b,), Numeric.Float)
	# print "b.shape=", b.shape
	assert a.n == b.shape[1]
	import lapack_dpb
	# print "type(a.d)==", type(a.d)
	# print "a.d=", a.d
	# print "ravel(a.d)=", Numeric.ravel(a.d)
	# print "a.ldab=", a.ldab
	result = lapack_dpb.dpbsv('L', a.n, a.kd, b.shape[0], a.d,
					a.ldab, b, max(1, a.n), 0)
	# print "result=", result
	if result['info'] != 0:
		raise NoSolutionError, 'Linear system has no solution. Lapack_dpb.dpbsv info code=%d' % result['info']
	# print "b=", b
	return b


def test1():
	x = sbd(100, 10)
	x[0,0] = 1
	assert x[0,0] == 1
	x[99,99] = 2
	assert x[99,99] == 2
	x[10,0] = 10
	assert x[10,0] == 10
	assert x[0,10] == 10
	try:
		x[12,1] = 2
	except IndexError:
		pass
	else:
		assert None


def err(a, b):
	return Numeric.sum(Numeric.ravel((a-b)**2))

def test2():
	x = sbd(6,2)
	b = Numeric.zeros((6,), Numeric.Float) + 10
	for i in range(6):
		x[i,i] = 2
	x[2,1] = 0.5
	# print "x=", x
	y = solve(x, b)
	y_t = Numeric.array(([5, 4, 4, 5, 5, 5],))
	# print "y=", y, "y_t=", y_t
	# print "x=", x
	assert err(y_t, y) < 1e-9


def testa():
	x = sbd(6, 2)
	x[4,3] = 1
	x[4,4] = 2
	x[5,3] = 3
	x[5,5] = -1

	y = x[:]
	# print "y=", y
	assert y[4,3]==1
	assert y[4,4]==2
	assert y[5,3]==3
	assert y[4,5]==0
	assert y[3,4]==1
	assert y[3,3]==0
	assert y[4,5]==0
	assert y[5,5] == -1
	y = x[5:6]
	# print "y=", y
	assert y[0,5] == -1
	assert y[0,3] == 3
	assert y[0,4] == 0

def testbdi():
	x = sbd(6,2)
	y = Numeric.array([[1, 2], [2, 3]])
	x.bd_increment(1, y)
	assert x[1,1]==1
	assert x[2,2]==3
	assert x[2,1]==2
	assert x[1,2]==2
	assert x[0,0]==0
	assert x[3,3]==0
	assert x[3,2]==0
	z = Numeric.array([[-1, 0, 1], [0, 0, 0], [0, 0, 0]])
	x.bd_increment(0, z)
	assert x[0,0]==-1
	assert x[0, 2]==1
	assert x[1,1]==1
	assert x[1, 2]==2

if __name__ == '__main__':
	test1()
	test2()
	testa()
	testbdi()



"""Reads label files produced by ESPS xwaves.
Writes labels files that ESPS xwaves can read.
"""

import os
import sys
from gmisclib import die
from gmisclib.xwaves_errs import *


def read(filename, loose=0):
	"""read in .lab files produced by ESPS xlabel. returns (header, data).
	Data = [(time, label), ...].
	Note that leading or trailing spaces in the label are removed.
	"""
	HUGE = 1e30
	hdr = {}
	if filename == '-':
		fd = sys.stdin
	else:
		try:
			fd = open(filename, "r")
		except IOError, x:
			raise NoSuchFileError, x
	n = 0
	comments = []
	while True:
		l = fd.readline()
		n += 1
		if not l:
			raise BadFileFormatError, '%s:%d' % (filename, n)
		if l == '#\n':
			break
		if l.startswith('comment'):
			comments.append(l[len('comment'):].strip())
			continue
		try:
			a, v = l.split(None, 1)
		except ValueError:
			if loose:
				hdr[l.strip()] = ''
				loose -= 1
			else:
				die.warn( ("Line %d:" % n) + l.strip())
				raise BadFileFormatError, '%s:%d' % (filename, n)
		else:
			hdr[a.strip()] = v.strip()

	t_last = -HUGE
	d = []
	while True:
		l = fd.readline()
		if not l:
			break
		n += 1

		l = l.lstrip().rstrip('\r\n')
		if not l:
			continue	# Skip blank lines

		lsplit = l.split(None, 2)

		if len(lsplit)==2 and (l.endswith(' ') or l.endswith('\t')):
			t, junk = lsplit
			label = ''
		elif len(lsplit)==2 and loose > 0:
			loose -= 1
			t,junk = lsplit
			label = ''
			comments.append('Two columns: assuming third is empty: line %d: %s' % (n, l))
		elif len(lsplit) == 2:
			raise BadFileFormatError, 'Two columns: should be three: %s:%d' % (filename, n)
		else:
			t, junk, label = lsplit

		t = float(t)
		if t >= t_last:
			d.append((t, label))
		elif loose > 0:
			d.append((t, label))
			loose -= 1
		else:
			raise DataOutOfOrderError, '%s:%d' % (filename, n)
		t_last = t

	fd.flush()
	os.fsync(fd.fileno())	# Commit to disk.
	fd = None	# This will close most files, except for sys.stdin
			# which has probably some other reference keeping
			# it open.

	hdr['_COMMENT'] = '\n'.join(comments)
	hdr['_NAME'] = filename
	hdr['_FILETYPE'] = 'xlabel'
	hdr['NAXIS'] = 2
	hdr['NAXIS2'] = len(d)
	hdr['NAXIS1'] = 2
	hdr['TTYPE1'] = 'time'
	hdr['TUNIT1'] = 's'
	hdr['TTYPE2'] = 'label'

	return (hdr, d)


def start_stop(d, dropfirst=False):
	"""Converts data in (end_time, label) format to (t_start, t_stop, label)"""
	o = []
	last = None
	for (time, label) in d:
		o.append( (last, time, label) )
		last = time
	if dropfirst and len(o)>0:
		o.pop(0)
	return o


def end_marks(d, beglabel, delta=0.001, interlabel=None):
	"""Converts data in a (start, stop, label) format to (end_time, label).
	It introduces extra labels if neccessary.
	A beginning label is required, to mark the start of the first segment."""
	if len(d) == 0:
		return []
	if interlabel is None:
		interlabel = beglabel
	start, stop, label = d[0]
	o = [ (start, beglabel), (stop, label) ]
	laststop = stop
	for (start, stop, label) in d[1:]:
		if start > laststop + delta:
			o.append( (start, interlabel) )
		o.append( (stop, label) )
		laststop = stop
	return o


def write(fd, header, data):
	"""Expects data in [(t, label), ...] form.
	"""
	h = {'font': '-misc-*-bold-*-*-*-15-*-*-*-*-*-*-*',
		'separator': ';',
		'nfields': ';'
		}
	h.update(header)
	d = list(data)
	d.sort()

	alist = h.items()
	alist.sort()
	for (a, v) in alist:
		if v != '':
			for tmp in str(v).split('\n'):
				fd.write('%s %s\n' % (a, tmp))
	fd.write('#\n')
	fd.flush()
	for (t, mark) in data:
		fd.write('%12.5f    -1  %s\n' % (t, str(mark).strip()))
	fd.flush()


if __name__ == '__main__':
	hdr, data = read(sys.argv[1])
	for (t, l) in data:
		print t, l




"""Markov-Chain Monte-Carlo algorithms.

Here, we do MCMC when -log(p) is a sum of squares."""

import mcmcSQ
import die

import mcmc
import g_implements
import findleak

_s = None

def go_step(x, showvec):
	global _s

	findleak.vm('go_step start')
	assert g_implements.impl(x.current(), mcmc.position_base), "Bad pseudoposition: %s" % g_implements.why(x.current(), mcmc.position_base)
	if _s is None:
		print "INITIALIZING mcmc"
		_s = mcmc.bootstepper(None, x.current(), x.archive.mvn.cov, None)
		findleak.vm('go_step initialize end')
	print "SQ STEP"
	accepted = x.step(showvec=showvec, atmisc={'stepper': 'SQ'})
	findleak.vm('go_step accepted start')
	if accepted:
		cur = x.current()
		print 'Cur from SQ:', cur.prms()
		_s._set_current( cur )
		findleak.vm('go_step accepted end')
	findleak.vm('go_step Bootstep start')
	if(len(_s.archive) > _s.np/_s.F):
		print "BOOT STEP"
		accepted = _s.step()
		print 'Accepted:', accepted
		cur = x.current()
		print 'Cur from BOOT:', cur.prms()
		SQpos = _s.current()
		assert isinstance(SQpos, mcmcSQ.position)
		x.archive.add( SQpos )
		if accepted:
			findleak.vm('go_step BOOTstep-set_current start')
			x._set_current( SQpos )
			findleak.vm('go_step BOOTstep-set_current end')
		x.print_at( misc = {'stepper': 'Boot', 'BootScale': _s.a[0].vs()} )
		findleak.vm('go_step BOOTstep end')
	#findleak.print_top_N(20)
	findleak.vm('go_step end')

mcmcSQ.go_step = go_step





if __name__ == '__main__':
	import sys
	if len(sys.argv) <= 1:
		print __doc__
		die.exit(0)
	mcmcSQ.go(sys.argv)



from gmisclib import Num
import gmisclib.Numeric_gpk as NG

class lls_base(object):
	def __init__(self, a, copy=True):
		self.ginv = None
		self._fit = None
		self._x = None
		self._hatdiag = None
		self._y = None
		self.a = Num.array(a, Num.Float, copy=copy)
		if len(self.a.shape) != 2:
			raise ValueError, "a needs to be 2-dimensional: shape=%s" % str(self.a.shape)
		self.m, self.n = self.a.shape
		self.q = None


	def set_y(self, y, copy=True):
		if y is None:
			return
		self._fit = None
		self._x = None
		self._y = Num.array(y, Num.Float, copy=copy)
		if len(self._y.shape) == 1:	# Vector
			self.vector = True
			self._y = Num.transpose( [y] )
		elif len(self._y.shape) > 2:
			raise ValueError, "y needs to be 1- or 2-dimensional: shape=%s" % str(self._y.shape)
		else:
			self.vector = False
		if self._y.shape[0] != self.m:
			raise ValueError, "Matrix sizes do not match: (%d,%d) and %s" % (
								self.m, self.n, str(self._y.shape)
								)
		self.q = self._y.shape[1]


	def y(self, copy=True):
		if self.vector:
			return Num.array(self._y[:,0], copy=copy)
		return Num.array(self._y, copy=copy)


	def _solve(self):
		raise RuntimeError, "Virtual Function"


	def hat(self, copy=True):
		raise RuntimeError, "Virtual Function"


	def x(self, y=None, copy=True):
		self.set_y(y)
		assert self._y is not None, "Y not yet set"
		if self._x is None:
			self._x = self._solve()
			if self.vector:	# Restore everything to vectors
				self._x = self._x[:, 0]
		return Num.array(self._x, copy=copy)


	def fit(self, copy=False):
		if self._fit is None:
			self._fit = Num.dot(self.a, self.x(copy=False))
		return Num.array(self._fit, copy=copy)


	def residual(self):
		# print 'resid shapes=', self.fit().shape, self._y.shape
		return self.y() - self.fit()


	def variance_about_fit(self):
		"""Returns the estimator of the standard deviation
		of the data about the fit.
		@return: L{numpy.ndarray} with shape=(q,).   Each entry corresponds
			to one of the C{q} sets of equations that are being fit.
		"""
		r2 = Num.sum(self.residual()**2, axis=0)
		assert self.vector and r2.shape==() or not self.vector and r2.shape==(self.q,)
		return r2/(self.m-self.n)


	def eff_rank(self):
		return Num.sum(1.0 - self.hat(copy=False))



class linear_least_squares(lls_base):
	def __init__(self, a, y=None, minsv=None, minsvr=None, copy=True):
		"""This solves the set of linear equations a*x = y,
		and returns (x, the_fit, rank, s).
		Normally, a.shape==(m,n) and y.shape==(m,q),
		and the returned x.shape==(n,q).
		where m is the number of constraints provided by the data,
		n is the number of parameters to use in a fit
		(equivalently, the number of basis functions),
		and q is the number of separate sets of equations
		that you are fitting.
		Then, x has shape (n,q) and the_fit has shape (m,q).
		Interpreting this as a linear regression, there are n
		parameters in the model, and m measurements.
		Q is the number of times you apply the model to a
		different data set; each on yields a different solution.
	
		The procedure uses a singular value decomposition algorithm,
		and treats all singular values that are smaller than minsv
		as zero (i.e. drops them). 
		If minsvr is specified, it treates all singular values that
		are smaller than minsvr times the largest s.v. as zero.
		The returned rank is the
		rank of the solution, which is normally the number of
		nonzero elements in x.
		Note that the shape of the solution vector or matrix
		is defined by a and y, and the rank can be smaller
		than m.
	
		Y may be a 1-D matrix (a vector), in which case
		the fit is a vector.    This is the normal
		case where you are fitting one equation.
		If y is a 2-D matrix,
		each column (second index) in y is a separate fit, and
		each column in the solution is a separate result.
		"""
	
		lls_base.__init__(self, a, copy=copy)
		self.set_y(y, copy=copy)

		assert minsv is None or minsv >= 0.0
		assert minsvr is None or 0.0 <= minsvr <= 1.0
		r = min(self.m, self.n)
		u, self.s, vh = Num.LA.svd(self.a)
		assert u.shape == (self.m, self.m)
		assert vh.shape == (self.n, self.n)
		assert self.s.shape == (r,)
		# rbasis = Num.dot(Num.dot(u, sigma), vh)
		# rbasis = Num.dot(u[:,:r]*self.s, vh[:r])
		# rbasis = Num.dot(u[:,:r], self.s*vh)
		# print 'rbasis=', rbasis
		# assert Num.sum(Num.ravel(rbasis-a)**2) < 0.0001*Num.sum(Num.ravel(self.a)**2)
		if minsv is not None and minsvr is not None:
			svrls = max(minsv, minsvr*self.s[Num.argmax(self.s)] )
		elif minsv is not None:
			svrls = minsv
		elif minsvr is not None:
			svrls = minsvr * self.s[Num.argmax(self.s)]
		else:
			svrls = 0.0
	
		self.sim = Num.greater(self.s, svrls)
		isi = Num.where(self.sim, 1.0/Num.where(self.sim, self.s, 1.0), 0.0)
		self.ginv = Num.transpose(Num.dot(u[:,:r]*isi, vh[:r]))

	def _solve(self):
		return Num.dot(self.ginv, self._y)

	def sv(self):
		return self.s

	def rank(self):
		return Num.sum(self.sim)

	def hat(self, copy=True):
		"""Hat Matrix Diagonal
		Data points that are far from the centroid of the X-space are potentially influential.
		A measure of the distance between a data point, xi,
		and the centroid of the X-space is the data point's associated diagonal
		element hi in the hat matrix. Belsley, Kuh, and Welsch (1980) propose a cutoff of
		2 p/n for the diagonal elements of the hat matrix, where n is the number
		of observations used to fit the model, and p is the number of parameters in the model.
		Observations with hi values above this cutoff should be investigated.
		For linear models, the hat matrix

		C{H = X inv(X'X) X'}

		can be used as a projection matrix.
		The hat matrix diagonal variable contains the diagonal elements
		of the hat matrix

		C{hi = xi inv(X'X) xi'}
		"""
		if self._hatdiag is None:
			aainv = Num.dot(self.ginv, Num.transpose(self.ginv))
			hatdiag = Num.zeros((self.a.shape[0],), Num.Float)
			for i in range(hatdiag.shape[0]):
				hatdiag[i] = NG.qform(self.a[i,:], aainv)
				assert -0.001 < hatdiag[i] < 1.001
			self._hatdiag = hatdiag
		return Num.array(self._hatdiag, copy=copy)


	def x_variances(self, copy=True):
		"""Estimated standard deviations of the solution.
		This is the diagonal of the solution covariance matrix.
		"""
		aainv = Num.dot(self.ginv, Num.transpose(self.ginv))
		vaf = self.variance_about_fit()
		rv = Num.outerproduct(aainv.diagonal(), vaf)
		assert rv.shape == self._x.shape
		assert rv.shape == (self.n, self.q)
		if self.vector:
			return rv[0,:]
		return rv




class reg_linear_least_squares(lls_base):
	def __init__(self, a, y, regstr=0.0, regtgt=0.0, rscale=None):
		"""This solves min! |a*x - y|^2 + |regstr*(x-regtgt)|^2,
		and returns (x, the_fit, rank, s).
		Normally, a.shape==(m,n) and y.shape==(m,q),
		where m is the number of data to be fit,
		n is the number of parameters to use in a fit
		(equivalently, the number of basis functions),
		and q is the number of separate sets of equations
		that you are fitting.
		Then, x has shape (n,q) and the_fit has shape (m,q).
	
		The regularization target,
		regtgt is the same shape as x, that is (n,q).
		(It must be a vector if and only if y is a vector.)
		Regstr, the strength of the regularization is
		normally an (n,n) matrix, though (*,n) will work,
		as will a scalar.
	
		Y may be a 1-D matrix (a vector), in which case
		the fit is a vector.    This is the normal
		case where you are fitting one equation.
		If y is a 2-D matrix,
		each column (second index) in y is a separate fit, and
		each column in the solution is a separate result.
		"""
		lls_base.__init__(self, a)
		self.set_y(y)

		if self.vector:
			if len(regtgt.shape) != 1:
				raise ValueError, "regtgt must be a vector if y is a vector."
			regtgt = Num.transpose( [regtgt] )
		self.regtgt = Num.array(regtgt, Num.Float, copy=True)
		if self.regtgt.shape[0] != self.n:
			raise ValueError, "Regtgt shape must match the shape of a"
		regstr = Num.asarray(regstr, Num.Float)
		assert len(regstr.shape) == 2
		assert regstr.shape[1] == self.n
	
		self.rr = Num.dot(Num.transpose(regstr), regstr)
		self.aa = Num.dot(Num.transpose(self.a), self.a)
		if rscale is None:
			self.scale = 1.0
		else:
			self.scale = rscale * Num.trace(self.aa)/Num.trace(self.rr)
		# print 'aa=', self.aa
		# print 'rr=', self.rr
		# print 'traa', Num.trace(self.aa)
		# print 'trrr', Num.trace(self.rr)
		# print 'scale=', self.scale
		self.aareg = self.aa + self.rr*self.scale


	def _solve(self):
		ayreg = Num.dot(Num.transpose(self.a), self._y)	\
				+ Num.dot(self.rr, self.regtgt)*self.scale
		return Num.LA.solve(self.aareg, ayreg)
	

	def sv_reg(self):
		"""Singular values of the regularized problem."""
		return Num.sqrt(Num.LA.eigvalsh(self.aareg))

	def sv_unreg(self):
		"""Singular values of the unregularized problem."""
		return Num.sqrt(Num.LA.eigvalsh(self.aa))

	def hat(self, copy=True):
		"""Hat Matrix Diagonal
		Data points that are far from the centroid of the X-space are potentially influential.
		A measure of the distance between a data point, xi,
		and the centroid of the X-space is the data point's associated diagonal
		element hi in the hat matrix. Belsley, Kuh, and Welsch (1980) propose a cutoff of
		2 p/n for the diagonal elements of the hat matrix, where n is the number
		of observations used to fit the model, and p is the number of parameters in the model.
		Observations with hi values above this cutoff should be investigated.
		For linear models, the hat matrix

		    H = X (X'X)-1 X' 

		    can be used as a projection matrix.
		    The hat matrix diagonal variable contains the diagonal elements
		    of the hat matrix

		        hi = xi (X'X)-1 xi' 
		"""
		if self._hatdiag is None:
			hatdiag = Num.zeros((self.aa.shape[0],), Num.Float)
			iaareg = Num.LA.inv(self.aareg)
			for i in range(hatdiag.shape[0]):
				hatdiag[i] = NG.qform(self.a[i,:], iaareg)
				assert -0.001 < hatdiag[i] < 1.001
			self._hatdiag = hatdiag
		return Num.array(self._hatdiag, copy=copy)



def test_svd():
	a0 = Num.array([[0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0],
		[0, 0, 0, 0, 0, 0, 0, -3, -2, -1, 0, 1, 2, 3, 0, 0, 0, 0, 0],
		[0, 0, 0, 0, 0, 0, 0, 1, -1, 1, -1, 1, -1, 1, 0, 0, 0, 0, 0]], Num.Float)
	a1 = Num.array([[1, 1, 1],
		[-3, -2, -1],
		[1, -1, 1]], Num.Float)
	a2 = Num.array([[1, 1, 1], [-3, -2, -1], [1, -1, 1], [0, 0, 2.2], [2, 0, 0], [0, 1, 0]],
			Num.Float)
	for a in [a0, a1, a2]:
		u, s, vh = Num.LA.svd(a)
		r = min(a.shape)
		# print 'shapes for u,s, vh=', u[:,:r].shape, s.shape, vh[:r].shape, 'r=', r
		assert Num.alltrue( s >= 0.0)
		err = Num.sum(Num.ravel(Num.dot(u[:,:r]*s, vh[:r]) - a)**2)
		assert err < 1e-6


def all_between(a, vec, b):
	return Num.alltrue(
			Num.greater_equal(vec, a)
			* Num.greater_equal(b, vec)
			)

def test_vec():
	basis = Num.transpose([[1, 2, 3, 4, 5, 6, 7, 8, 9, 10], [1, 1, 1, 1, 1, 1, 1, 1, 1, 1]])
	y = Num.array([0, 1, 2, 3, 4, 5, 6, 7, 8, 9], Num.Float)
	soln = linear_least_squares(basis, y, 1e-6)
	hat = soln.hat()
	assert Num.alltrue(hat > 1.0/y.shape[0])
	assert Num.alltrue(hat < 4.0/y.shape[0])
	assert soln.rank() == 2
	print 'fitshape=', soln.fit().shape, y.shape
	err = Num.sum((soln.fit()-y)**2)
	assert 0.0 <= err < 1e-6
	print 'soln.residual=', soln.residual()
	assert all_between(-1e-6, soln.residual(), 1e-6)
	assert soln.sv().shape == (2,)
	assert abs(soln.x()[0] - 1) < 1e-6
	assert abs(soln.x()[1] + 1) < 1e-6
	assert Num.sum(Num.absolute(Num.dot(basis, soln.x()) - soln.fit())) < 1e-6


def test_vec2():
	basis = Num.transpose([[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
				[1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]])
	y = Num.array([0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0], Num.Float)
	# This test is obsolete and needs to be fixed.
	soln = linear_least_squares(basis, y, 1e-6)
	assert soln.rank() == 2
	err = Num.sum((soln.fit()-y)**2)
	avg = 10.0/11.0
	epred = 6.0*avg**2 + 5.0*(2-avg)**2
	assert soln.sv().shape == (2,)
	assert abs(soln.x()[1] - avg) < 1e-6
	assert abs(soln.x()[0]) < 1e-6
	assert abs(err - epred) < 1e-4



def test_m1():
	basis = Num.transpose([[1, 2, 3, 4, 5, 6, 7, 8, 9, 10], [1, 1, 1, 1, 1, 1, 1, 1, 1, 1]])
	y = Num.transpose([[0, 1, 2, 3, 4, 5, 6, 7, 8, 9]])
	assert y.shape[1] == 1
	soln = linear_least_squares(basis, y, 1e-6)
	assert soln.x().shape[1] == 1
	assert soln.fit().shape[1] == 1
	assert soln.rank() == 2
	err = soln.residual()**2
	assert all_between(0, err, 1e-6)
	assert soln.sv().shape == (2,)
	assert abs(soln.x()[0,0] - 1) < 1e-6
	assert abs(soln.x()[1,0] + 1) < 1e-6
	assert Num.sum(Num.absolute(Num.ravel(Num.dot(basis, soln.x())-y))) < 1e-6


def test_m2():
	basis = Num.transpose([[1, 2, 3, 4, 5, 6, 7, 8, 9, 10], [1, 1, 1, 1, 1, 1, 1, 1, 1, 1]])
	y = Num.transpose([[0, 1, 2, 3, 4, 5, 6, 7, 8, 9], [9, 8, 7, 6, 5, 4, 3, 2, 1, 0]])
	assert y.shape[1] == 2
	soln = linear_least_squares(basis, y, 1e-6)
	assert soln.x().shape[1] == 2
	assert soln.fit().shape[1] == 2
	assert soln.rank() == 2
	err = Num.sum((soln.fit()-y)**2, axis=0)
	assert err.shape[0]==2
	assert all_between(0.0, err, 1e-6)
	assert soln.sv().shape == (2,)
	assert abs(soln.x()[0,0] - 1) < 1e-6
	assert abs(soln.x()[1,0] + 1) < 1e-6
	assert abs(soln.x()[0,1] + 1) < 1e-6
	assert abs(soln.x()[1,1] - 10) < 1e-6
	assert Num.sum(Num.absolute(Num.ravel(Num.dot(basis, soln.x())-y))) < 1e-6


def test_m2r():
	basis = Num.transpose([[1, 2, 3, 4, 5, 6, 7, 8, 9, 10], [1, 1, 1, 1, 1, 1, 1, 1, 1, 1]])
	y = Num.transpose([[0, 1, 2, 3, 4, 5, 6, 7, 8, 9], [9, 8, 7, 6, 5, 4, 3, 2, 1, 0]])
	assert y.shape[1] == 2
	soln = reg_linear_least_squares(basis, y, Num.zeros((2,2), Num.Float), [[0.0, 0.0], [0.0, 0.0]])
	hat = soln.hat()
	assert Num.alltrue(hat > 1.0/y.shape[0])
	assert Num.alltrue(hat < 4.0/y.shape[0])
	assert soln.x().shape[1] == 2
	assert soln.fit().shape[1] == 2
	err = Num.sum((soln.fit()-y)**2, axis=0)
	assert err.shape[0]==2
	assert all_between(0.0, err, 1e-6)
	assert abs(soln.x()[0,0] - 1) < 1e-6
	assert abs(soln.x()[1,0] + 1) < 1e-6
	assert abs(soln.x()[0,1] + 1) < 1e-6
	assert abs(soln.x()[1,1] - 10) < 1e-6
	assert Num.sum(Num.absolute(Num.ravel(Num.dot(basis, soln.x())-y))) < 1e-6


def test_m2rR():
	basis = Num.transpose([[1, 2, 3, 4, 5, 6, 7, 8, 9, 10], [1, 1, 1, 1, 1, 1, 1, 1, 1, 1]])
	y = Num.transpose([[0, 1, 2, 3, 4, 5, 6, 7, 8, 9], [9, 8, 7, 6, 5, 4, 3, 2, 1, 0]])
	assert y.shape[1] == 2
	soln = reg_linear_least_squares(basis, y, 1e6*Num.identity(2), [[0.5, 0.5],[0.5,0.5]])
	assert soln.x().shape[1] == 2
	assert soln.fit().shape[1] == 2
	err = Num.sum((soln.fit()-y)**2, axis=0)
	assert err.shape[0]==2
	assert all_between(1.0, err, 500.0)
	assert abs(soln.x()[0,0] - 0.5) < 1e-3
	assert abs(soln.x()[1,0] - 0.5) < 1e-3
	assert abs(soln.x()[0,1] - 0.5) < 1e-3
	assert abs(soln.x()[1,1] - 0.5) < 1e-3
	soln = reg_linear_least_squares(basis, y, 1e6*Num.identity(2), [[0.5, 0.5],[0.5,0.5]], rscale=1.0)
	print 's', soln.x()
	print 'f', soln.fit()
	print 'r', soln.eff_rank()


def test_hat():
	"""Make sure that the hat function meets its definition."""
	basis = Num.transpose([[1, 2, 3, 4, 5, 6, 7, 8, 9, 10], [1, 1, 1, 1, 1, 1, 1, 1, 1, 1]])
	y0 = Num.transpose([0, 1, 2, 3, 4, 5, 6, 7, 8, 9])
	s0 = linear_least_squares(basis, y0)
	assert y0.shape[0] > 1
	for i in range(y0.shape[0]):
		y = Num.array(y0)
		y[i] += 1
		s = linear_least_squares(basis, y)
		ishift = s.fit()[i] - s0.fit()[i]
		assert -0.001 <= ishift <= 1.001
		assert abs(ishift - s0.hat()[i]) < 0.001


if __name__ == '__main__':
	test_svd()
	test_vec()
	test_m1()
	test_m2()
	test_vec2()
	test_m2r()
	test_m2rR()
	test_hat()



"""Run as an independent program, this prints out the information
in a .out file, generated for/by ESPS Xmark.
If you give it a -a flag, it will print just a single attribute from the
header of the Xmark file.

For example:

xwaves_mark.py -a utterance

returns the top line, which is normally the transcribed utterance.
"""

import os
import sys
from gmisclib import die
import types
from gmisclib.xwaves_errs import *

DecrTol = 0.004999	# The maximum amount that time marks can decrease
			# before it is considered an error.


def _monotonize_tail(d):
	i = len(d)-1
	t = d[-1][0]
	sum = 0.0
	n = 0
	while i>=0 and isinstance(d[i], types.TupleType):
		if d[i][0] < t:
			break
		sum += d[i][0]
		n += 1
		i -= 1
	avg = sum/float(n)
	for j in range(i+1, len(d)):
		tt, label, ty = d[j]
		d[j] = (avg, label, ty)





PHONE = 1
WORD = 0

def write(fd, hdr, data, SortData=1):
	cs = hdr.get('_COMMENT', '').split('\n')
	for c in cs:
		fd.writelines('# %s\n' % c)
	for (k, v) in hdr.items():
		if k != '_COMMENT' and k != 'utterance':
			fd.writelines('%s %s\n' % (k, v))
	if hdr.has_key('utterance'):
		fd.writelines('** %s\n' % hdr['utterance'])
	else:
		fd.writelines('**\n')

	if SortData:
		tmp = data[:]
		tmp.sort()
		data = tmp

	last = None
	for (t, lbl, ty) in data:
		if t < last:
			raise DataOutOfOrderError((t, lbl, ty))
		if ty == WORD:
			fd.writelines('%s\n' % lbl)
		else:
			fd.writelines('\t%s\t%f\n' % (lbl, t))
		last = t



def read(filename):
	"""Read in .in files produced by ESPS xmark.
	Returns (header, data),
	where data is (time, word_or_phoneme, type),
	where type is 0 for words, 1 for phonemes.
	Times are guarenteed to be increasing inside the set of
	all words, and also inside the set of all phonemes.
	Word marks preceed the corresponding phoneme marks.
	"""

	hdr = {}
	if filename == '-':
		fd = sys.stdin
	else:
		try:
			fd = open(filename, "r")
		except IOError, x:
			raise NoSuchFileError(x)

	# First, we read the header:
	comments = []
	n = 0
	while True:
		l = fd.readline()
		n += 1
		if l == '':
			raise RuntimeError, 'Premature EOF / bad file format: %s:%d' % (filename, n)
		if l.startswith('#'):
			comments.append(l[1:].strip())
			continue
		if l.startswith('**'):
			hdr['utterance'] = l[2:].strip()
			# Header is terminated by a line starting '**'.
			break
		try:
			a, v = l.split(None, 1)
			# Header can contain attribute/value pairs.
		except ValueError:
			die.warn("Line %d:"%n + l)
			raise BadFileFormatError, '%s:%d' % (filename,n)
		hdr[a.strip()] = v.strip()

	# Now, we read in the data:
	d = []
	t_last = -1e30
	while True:
		l = fd.readline()
		n += 1
		if not l:		# EOF
			break
		ls = l.strip()
		if not ls:		# Ignore blank lines.
			continue
		if l[0].isspace():	# Segmentation information is indented.
			try:
				label, t = ls.split(None, 2)
			except ValueError:	# Only one thing on the line.
				die.warn("Incomplete file: File: %s, Line %d:(%s)"%(filename, n, ls))
				# This is not a fatal problem at the end, especially not
				# if the last label is silence.
				tmp = fd.readline()
				if tmp == '':	# OK.  Incomplete was last line.
					break
				raise BadFileFormatError, '%s:%d' % (filename, n)
			t = float(t)
			if t >= t_last:
				d.append((t, label, PHONE))	# Put a tuple on the list.
			elif t >= t_last - DecrTol:
				d.append((t, label, PHONE))	# Put a tuple on the list.
				_monotonize_tail(d)		# Force the times to be monotonic.
				die.warn("Time decreases slightly: %s:%d" % (filename, n))
			else:
				raise DataOutOfOrderError, 'time is decreasing: %s:%d' % (filename, n)
			t_last = t
		else:		# Words are not indented.
			d.append(ls)				# Put a string on the list.

	fd.flush()
	os.fsync(fd.fileno())	# Commit to disk.
	fd = None	# This will close the file descriptor unless
			# (as in the case of sys.stdin) something else
			# is holding a reference.

	# Next, we need to add a null word to the beginning.  Recall that we report ending
	# times: without a null word, we only know when the first word ended, not when
	# it started.  It starts when the null word ends, of course.
	if len(d)>0 and isinstance(d[0], types.TupleType):
		d.insert(0, '')

	# Now, we go through on a second pass, and add timing information to the words.
	# We assume that the word comes first, then the phonemes into which it is segmented.
	defer = None		# Marking the position of the last word seen.
	last_t = None
	for i in range(len(d)):
		if isinstance(d[i], str):	# Word -- just the string.
			if defer is not None:
						# Go back, and fix up the previous word,
						# now that we have the ending time.
				d[defer] = (last_t, d[defer], WORD)
			defer = i
		else:		# Tuple: phoneme (segmentation)
			last_t = d[i][0]	# Remember the ending time of the phoneme.

	if defer is not None:
		d[defer] = (last_t, d[defer], WORD)

	hdr['_COMMENT'] = '\n'.join(comments)
	hdr['_NAME'] = filename
	hdr['_FILETYPE'] = 'xmark'
	hdr['NAXIS'] = 2
	hdr['NAXIS2'] = len(d)
	hdr['NAXIS1'] = 3
	hdr['TTYPE1'] = 'time'
	hdr['TUNIT1'] = 's'
	hdr['TTYPE2'] = 'label'
	hdr['TTYPE3'] = 'is_phoneme'

	return (hdr, d)


def mark_to_lab(data, ty):
	"""This function lets you take a mixed list
	of word and phone labels, such as provided
	by xwaves_mark.read(), and will select out
	one or the other type.
	Type is PHONE or WORD.
	"""
	return [(time, label) for (time, label, typ) in data if typ==ty ]



def combine_2labs(whdr, wd, phdr, pd, utterance, TOL=0.001):
	"""Combine two XLAB files into one XMARK file.
	It forces (within rounding errors) the words
	to enclose the phones."""

	import xwaves_lab

	hdr = phdr.copy()
	hdr.update(whdr)
	hdr['utterance'] = utterance

	dw = xwaves_lab.start_stop(wd, dropfirst=1)
	dp = xwaves_lab.start_stop(pd, dropfirst=1)

	o = []
	j = 0
	o.append( ( dp[0][0], '*', PHONE) )
	# print "PRE S - %f *" % dp[0][0]
	while j < len(dp) and dp[j][1] < dw[0][0]-TOL:
		# print "PRE S %d %f %f %s" % (j, dp[j][0], dp[j][1], dp[j][2])
		o.append( ( dp[j][1], dp[j][2], PHONE) )
		j += 1
	for (start, stop, w) in dw:
		# print "w %f %f %s" % (start, stop, w)
		w_appended = 0
		while j<len(dp) and dp[j][1]<stop+TOL and dp[j][0]<stop:
			if not w_appended and dp[j][2] != '*':
				# print "W %f %f %s" % (dp[j][0], stop, w)
				o.append( (dp[j][0], w, WORD) )
				w_appended = 1
			# print "IN S %d %f %f %s" % (j, dp[j][0], dp[j][1], dp[j][2])
			o.append( (dp[j][1], dp[j][2], PHONE) )
			j += 1
		if not w_appended:
			o.append( ( start, w, WORD) )
	while j < len(dp):
		# print "POST S %d %f %f %s" % (j, dp[j][0], dp[j][1], dp[j][2])
		o.append( (dp[j][1], dp[j][2], PHONE) )
		j += 1
	return (hdr, o)


__doc__ = read.__doc__

if __name__ == '__main__':
	arglist = sys.argv[1:]
	if arglist[0] == '-a':
		arglist.pop(0)
		key = arglist.pop(0)
		hdr, data = read(arglist.pop(0))
		print hdr[key]
	elif arglist[0] == '-tolab':
		import xwaves_lab
		arglist.pop(0)
		ty = int(arglist.pop(0))
		hdr, data = read(arglist.pop(0))
		xwaves_lab.write(sys.stdout, hdr, mark_to_lab(data, ty))
	elif arglist[0] == '-fromlab':
		import xwaves_lab
		arglist.pop(0)
		whdr, wd = xwaves_lab.read(arglist.pop(0))
		phdr, pd = xwaves_lab.read(arglist.pop(0))
		hdr, data = combine_2labs(whdr, wd, phdr, pd,
					whdr.get('utterance', phdr.get('utterance', '')),
					TOL=0.001)
		write(sys.stdout, hdr, data, SortData=1)
	else:
		print read(sys.argv[1])




import os
import sys
import math

try:
	from Numeric_gpk import N_maximum, N_minimum, N_median, N_mean_ad,	\
			median_across, variance, stdev,	\
			set_diag,	\
			make_diag, limit, vec_variance, qform,	\
			KolmogorovSmirnov, interp, interpN
except ImportError, _x:
	if str(_x).startswith('cannot import name'):
		raise
	pass


def median(x):
	xx = sorted(x)
	n = len(xx)
	assert n > 0, "No data to median."
	return 0.5*(xx[n//2] + xx[(n-1)//2])


def avg(x):
	s = 0.0
	n = 0
	for t in x:
		s += t
		n += 1
	return s/float(n)


def median_ad(x):
	"""Median absolute deviation"""
	medn = median(x)
	return median( [ abs(t-medn) for t in x ] )
mad = median_ad


def mean_ad(x):
	medn = median(x)
	sum = 0.0
	for t in x:
		sum += abs(t-medn)
	return sum/float(len(x))


def geo_mean(*d):
	"""Geometric mean of its arguments."""
	s = 0.0
	for t in d:
		s += math.log(t)
	return math.exp(s/len(d))



def entropy(x):
	"""Compute the entropy of a list of probabilities."""
	e = 0.0
	ps = 0.0
	for p in x:
		assert p <= 1.0
		if p > 0.0:
			e -= p*math.log(p)
		ps += p
	assert 0.999 < ps < 1.001, "Probabilities must sum to one."
	return e


def resample(d):
	"""Bootstrap resampling.  Call this many times: each one
	returns a random resampling.
	"""
	import random
	o = []
	for i in range(len(d)):
		o.append( random.choice(d) )
	return o


def jackknife(d):
	"""Jackknife resampling.  Call this once.
	It returns a list of deleted lists."""
	for drop in range(len(d)):
		yield [ di for (i, di) in enumerate(d) if i!=drop ]






def Student_t_dens(x, n):
	"""From p.337 Statistical Theory by Bernard Lindgren."""

	from transcendental import gamma
	# http://bonsai.ims.u-tokyo.ac.jp/~mdehoon/software/python/statistics.html	
	p = gamma((n+1)/2.0) * (1+x*x/n)**(-(n+1)/2.0) / (
			math.sqrt(n*math.pi)*gamma(n/2.0)
			)
	return p




_fcache = {}
def log_factorial(n):
	assert n >= 0
	try:
		lf = _fcache[n]
	except KeyError:
		lf = 0.0
		for i in range(2,n):
			lf += math.log(i)
		_fcache[n] = lf
	return lf


def log_Combinations(n, m):
	assert n >= m
	assert m >= 0
	return log_factorial(n) - log_factorial(m) - log_factorial(n-m)

def ComplexMedian(P):
	"""P is a list of complex numbers.
	This algorithm works by repeatedly stripping off the convex
	hull of the points.
	"""
	import convex_hull2d
	import cmath
	import dictops
	HUGE = 1e30
	EPS = 1e-7
	Q = dictops.dict_of_accums()
	for p in P:
		Q.add(p, 1.0)

	while len(Q) > 3:
		# print 'Q=', Q
		edge = convex_hull2d.convexHull(Q.keys())
		# print 'edge=', edge
		ee = (edge[-1],) + edge + (edge[0],)
		wt = {}
		for i in range(1,len(edge)+1):
			em = ee[i] - ee[i-1]
			ep = ee[i+1] - ee[i]
			# angle = cmath.log(em).imag - cmath.log(ep).imag
			if min(abs(ep), abs(em)) < EPS*max(abs(em), abs(ep)):
				angle = math.pi/2.0	#Kluge, mild.   Roundoff errors.
			else:
				angle = cmath.log(em/ep).imag
			# KLUGE AND ROUNDOFF WARNING!
			if angle <= 0.0 and angle > -0.5:
				angle = EPS	#KLUGE!  Awful!
			if angle < 0.0:
				angle += 2*math.pi
			if angle >= math.pi and angle<math.pi+0.5:
				angle = math.pi-EPS	# KLUGE!  Awful!
			# print 'angle=', angle, ee[i-1], ee[i], ee[i+1], 'pi=', math.pi
			assert angle>=0 and angle<=math.pi, "angle=%g" % angle
			wt[ee[i]] = angle
		fmin = HUGE
		for p in edge:
			f = Q[p]/wt[p]
			if f < fmin:
				fmin = f
		# print 'fmin=', fmin
		assert fmin > 0.0
		sum = complex()
		swt = 0.0
		for p in edge:
			fwp = fmin * wt[p]
			# print 'Subtracting wt of ', fwp, 'from Q[', p, ']=', Q[p]
			swt  += fwp
			sum += p * fwp
			if Q[p] > fwp+EPS:
				Q[p] -= fwp
				# print '\tQ[', p, ']=', Q[p]
			else:	# Q[p]<=fwp
				assert abs(Q[p]-fwp) < EPS
				# print 'Deleting Q[', p, ']=', Q[p]
				del Q[p]
		if len(Q) == 0:
			return sum/swt
	# print 'Qfinal=', Q
	sum = complex()
	w = 0.0
	for (p,n) in Q.items():
		sum += p*n
		w += n
	return sum/w



def testCM():
	def eq(a, b):
		tmp = abs(a-b)/(abs(a)+abs(b)) < 1e-6
		if not tmp:
			print 'eq fails: %s vs %s' % (str(a), str(b))
		return tmp
	print 'CM'
	assert eq(ComplexMedian([complex(1,0), complex(2,0), complex(3,0)]), 2)
	assert eq(ComplexMedian([complex(1), complex(2), complex(3), complex(4)]), 2.5)
	assert eq(ComplexMedian([complex(1), complex(2), complex(3), complex(4), complex(5)]), 3)
	assert eq(ComplexMedian([complex(1), complex(2), complex(3), complex(4), complex(4)]), 3)
	assert eq(ComplexMedian([complex(1,1), complex(2,2), complex(3,3), complex(4,4)]),
			complex(2.5,2.5))
	assert eq(ComplexMedian([complex(0,0), complex(1,0), complex(0,1), complex(1,1)]),
			complex(0.5,0.5))
	assert eq( ComplexMedian([complex(0,0), complex(1,0), complex(0,1),
				complex(1,1), complex(1,1)]),
			complex(1,1))
	assert eq( ComplexMedian([complex(0,0), complex(1,0), complex(0,1),
				complex(1,1), complex(0.6,0.6)]),
			complex(0.6,0.6))
	assert eq(ComplexMedian([complex(0,0), complex(1,0), complex(0,1), complex(1,1),
				complex(0,0), complex(2,0), complex(0,2), complex(2,2)]),
			complex(0.5,0.5))
	assert eq(ComplexMedian([complex(0,0), complex(1,0), complex(0,1)]),
			complex(1./3., 1./3.))
	assert eq(ComplexMedian([complex(0,0), complex(1,0), complex(0,1),
				complex(0,0), complex(2,0), complex(0,2)]),
			complex(0.3, 0.3))
	import cmath
	for N in [3, 4, 5, 6, 7, 8, 13, 40, 100, 2351]:
		print 'N=', N
		assert eq(ComplexMedian( [ 1+cmath.exp(2*cmath.pi*1j*float(q)/N)
						for q in range(N) ]),
				complex(1.0, 0.0)
				)





import Queue
import threading
class threaded_readable_file(object):
	QSIZE = 100
	_string = type('')

	def __init__(self, fd):
		self.q = Queue.Queue(self.QSIZE)

		def rhelper(fd, q):
			try:
				for l in fd:
					q.put(l)
				q.put(None)
				# while True:
					# l = fd.readline()
					# if l == '':
						# q.put(None)
						# break
					# else:
						# q.put(l)
						
			except (Exception, KeyboardInterrupt):
				q.put( sys.exc_info() )
	
		self.rh = threading.Thread(target=rhelper, args=(fd, self.q))
		self.rh.start()


	def readline(self):
		if self.q is None:
			return ''
		x = self.q.get()
		if type(x) != self._string:
			self.rh.join()
			self.q = None
			if x is not None:
				raise x[0], x[1], x[2]
			return ''
		return x


	def readlines(self):
		o = []
		while self.q is not None:
			x = self.q.get()
			if type(x) != self._string:
				self.rh.join()
				self.q = None
				if x is not None:
					raise x[0], x[1], x[2]
				break
			o.append( x )
		return o


	def read_iter(self):
		while self.q is not None:
			x = self.q.get()
			if type(x) != self._string:
				self.rh.join()
				self.q = None
				if x is not None:
					raise x[0], x[1], x[2]
				break
			yield x

	__iter__ = read_iter


def thr_iter_read(fd):
	"""Read the contents of a file as an iterator.
	The read is two-threaded, so that one thread can be
	waiting on disk I/O while the other thread is
	processing the results.
	"""
	x = threaded_readable_file(fd)
	return x.read_iter()



def makedirs(fname, mode=0775):
	"""This makes the specified directory, including all
	necessary directories above it.    It is like os.makedirs(),
	except that if the directory already exists, it
	does not raise an exception.
	"""
	try:
		os.makedirs(fname, mode)
	except OSError, x:
		if os.path.isdir(fname) and os.access(fname, os.F_OK):
			return
		raise



def shuffle_Nrep(y, n=1, cmp=None):
	"""Shuffle a list, y,
	so that no item occurs more than n times in a row.
	Equality is determined by the comparison function cmp returning zero.
	"""
	import random
	assert n>0, "Silly!"
	x = y[:]
	random.shuffle(x)
	m = len(x)
	if cmp is None:
		cmp = lambda a, b: a==b

	passes = 0
	restart = 0
	while passes<1000:
		prb = None
		pstart = 0
		reps = 0
		# print 'x=', x
		for i in range(max(1, restart), m):
			if cmp(x[i-1], x[i]):
				reps += 1
				pstart = i - 1
			else:
				pstart = None
				reps = 0
			if reps >= n:
				prb = i
				break
		if prb is None:
			y[:] = x
			return y

		k = prb+1
		found = False
		while k < m:
			if not cmp(x[pstart], x[k]):
				found = True
				break
			k += 1
		if not found:
			# print 'ROLL'
			tmp = x.pop(0)
			x.append(tmp)
			restart = 0
			continue
		a = pstart
		b = k + 1
		# print 'a,b=', a, b
		tmp = x[a:b]
		random.shuffle(tmp)
		# print 'tmp=', x[a:b], '->', tmp
		x[a:b] = tmp
		restart = a
		passes += 1
	raise RuntimeError, 'Too many passes: cannot avoid repetitions'


def testSNR():
	import random
	for i in range(20):
		x = [ i//10 for i in range(20) ]
		shuffle_Nrep(x, 1)
		for i in range(len(x)-1):
			assert x[i] != x[i+1], 'Whoops: i=%d, %d %d' % (i, x[i], x[i+1])
		x = [ i//3 for i in range(10000) ]
		random.shuffle(x)
		shuffle_Nrep(x, 1)
		for i in range(len(x)-1):
			assert x[i] != x[i+1], 'Whoops: i=%d, %d %d' % (i, x[i], x[i+1])


def open_nowipe(nm1, nm2, mode="w"):
	"""Open a file (typically for writing) but make sure that the
	file doesn't already exist.   The name is constructed from
	nm1 a sequence number and nm2.   The sequence number gets incremented
	until a name is found that doesn't exist.
	Typically, nm2 is a suffix like ".wav"
	@param nm1: the part of the name to the left of the sequence number
	@param nm2: the part of the name to the right of the sequence number
	@param mode: The way to open the file -- passed to open().
	@type nm1: str
	@type nm2: str
	@type mode: str
	"""
	i = 0
	try:
		while True:
			nm = '%s%d%s' % (nm1, i, nm2)
			open(nm, 'r')
			i += 1
	except IOError:
		pass
	return (open(nm, mode), nm)


def dropfront(prefix, s):
	"""Drops the prefix from the string and raises an exception
	if it is not there to be dropped.
	"""
	if not s.startswith(prefix):
		raise ValueError, "String '%s' must start with '%s'" % (s[:20], prefix)
	return s[len(prefix):]


def open_compressed(fn):
	import g_pipe

	if fn.endswith('.bz2'):
		ci, co = g_pipe.popen2("bzcat", ['bzcat', fn])
		ci.close()
		return co
	elif fn.endswith('.gz'):
		ci, co = g_pipe.popen2("zcat", ['zcat', fn])
		ci.close()
		return co
	return open(fn, 'r')


def gammaln(x):
    """
Returns the gamma function of xx.
Gamma(z) = Integral(0,infinity) of t^(z-1)exp(-t) dt.
(Adapted from: Numerical Recipies in C. via code
Copyright (c) 1999-2000 Gary Strangman and released under the LGPL.)
"""
    coeff = [76.18009173, -86.50532033, 24.01409822, -1.231739516,
             0.120858003e-2, -0.536382e-5]
    tmp = x + 4.5
    tmp -= (x-0.5)*math.log(tmp)
    ser = 1.0
    for cj in coeff:
        ser += cj/x
        x += 1
    return -tmp + math.log(2.50662827465*ser)


def log_t_density(x, nu):
	c = gammaln((nu+1)/2.0) - 0.5*math.log(nu*math.pi) - gammaln(nu/2.0)
	return math.log(1+x**2/nu)*(-(nu+1)/2.0)



_a_factor_cache = {1:1, 2:2, 3:3}	# THIS CACHE SHOULD BE LOCKED
def a_factor(n):	# Don't mess with private_start.
	"""Finds the smallest prime factor of a number."""

	try:
		return _a_factor_cache[n]
	except KeyError:
		pass
	for p in primes():
		if p*p > n:
			f = n
			break
		if n%p == 0:
			f = p
			break
	_a_factor_cache[n] = f
	return f


_primes = [2, 3]	# 2, 3 *must* be there.	# THIS CACHE SHOULD BE LOCKED
def primes():
	"""This is a generator that produces an infinite list of primes.
	"""
	for p in _primes:
		yield p
	i = _primes[-1] + 2
	while True:
		for p in primes():
			if p*p > i:
				if p > _primes[-1]:	# multithreading...
					_primes.append(i)
				yield i
			if i%p == 0:
				i += 2
				break


_factor_cache = {}	# THIS CACHE SHOULD BE LOCKED
def factor(n):	# Don't mess with private_start.
	"""Factor a number into a list of prime factors,
	in increasing order.
	@param n: input number
	@type n: int
	@return: prime factors
	@rtype: list
	"""

	try:
		return _factor_cache[n]
	except KeyError:
		pass
	f = a_factor(n)
	if f == n:
		tmp = [n]
	else:
		tmp = [f] + factor(n//f)
	_factor_cache[n] = tmp[:]
	return tmp


def test_primes():
	assert factor(11) == [11]
	assert factor(16) == [2, 2, 2, 2]
	assert factor(100) == [2, 2, 5, 5]
	assert factor(2) == [2]
	assert factor(97) == [97]
	assert factor(55) == [5, 11]
	assert gcd(5,3)==1
	assert gcd(14, 21)==7
	assert gcd(100, 70)==10

def gcd(a, b):
	"""Greatest common factor/denominator.
	@type a: int
	@type b: int
	@rtype: int
	@return: the greatest common factor of a and b.
	"""
	assert a >= 0 and b >= 0
	while b != 0:
		tmp = b
		b = a % b
		a = tmp
	return a



def find_in_PATH(progname):
	"""Search PATH to find where a program resides.
	@param progname: the program to look for.
	@type progname: str
	@return: the full path name.
	@rtype: str
	"""
	for p in os.environ['PATH'].split(':'):
		tmp = os.path.join(p, progname)
		if os.access(tmp, os.R_OK|os.X_OK):
			return tmp
	return None


if __name__ == '__main__':
	testCM()
	testSNR()
	test_primes()



#!python

# This is a fancy installer that will download some missing packages.
# It's pretty good, so long as you have setuptools in your python distribution.

_imported = False
try:
	from setuptools import setup
	_imported = True
	pass
except ImportError:
	pass

#If it doesn't work, you can try the line below.   This uses the
# easy_setup.py script to download and install setuptools.
if not _imported:
	print "EZ_setup"
	try:
		from ez_setup import use_setuptools
		use_setuptools()
		from setuptools import setup
		_imported = True
		pass
	except ImportError:
		pass

# Finally, use distutils.   That should work, but you'll have to download
# all the required packages by hand.
if not _imported:
	from distutils.core import setup


setup(name = "gmisclib", version = "0.60.2",
	description = "Various Python Libraries, mostly scientific purposes",
	author = "Greg Kochanski",
	url = "http://kochanski.org/gpk/code/speechresearch/gmisclib",
	author_email = "gpk@kochanski.org",
	packages = ['gmisclib'],
	package_dir = {'gmisclib': '.'},
	scripts = ["bin/select_fiat_entries.py", 'bin/gpk_wavio.py'],
	install_requires = [
				# 'numpy >= 1.0.3',
				],
	extras_require = {'trimmed_mean': 'gpk_avg_py >= 1.0',
				'IO': 'gpk_img_python >= 1.2'},
	dependency_links = [
				],
	license = 'GPL2',
	keywords = "phonetics speech computational linguistics basic library python science optimize",
	platforms = "All",
	classifiers = [
		'License :: OSI Approved :: GNU General Public License (GPL)',
		'Operating System :: OS Independent',
		'Development Status :: 3 - Alpha',
		'Topic :: Scientific/Engineering',
		'Intended Audience :: Science/Research',
		'Programming Language :: Python',
		# X11
		'Operating System :: OS Independent',
		'Programming Language :: Python',
		],
	long_description = """Various Python Libraries, mostly scientific purposes.
	See http://kochanski.plus.com/code/speechresearch/gmisclib for documentation
	and http://sourceforge.net/projects/speechresearch for downloads,
	or look on the Python Cheese shop, http://pypi.python.org .
	"""
	)

